@@ -1,3 +1,5 @@
+# @describe All kinds of @option and @flag
+
 # @meta combine-shorts
 
 # @cmd All kind of options
@@ -7,6 +9,7 @@
 # @option    --oc!                  required
 # @option    --od*                  multi-occurs
 # @option    --oe+                  required + multi-occurs
+# @option    --of*,                 multi-occurs + comma-separated list
 # @option    --ona <PATH>           value notation
 # @option    --onb <FILE> <FILE>    two-args value notations
 # @option    --onc <CMD> <FILE+>    unlimited-args value notations
@@ -15,14 +18,13 @@
 # @option    --oca[a|b]             choice
 # @option    --ocb[=a|b]            choice + default
 # @option    --occ*[a|b]            multi-occurs + choice
-# @option    --ocd+[a|b]            required + multi-occurs + choice
 # @option    --ofa[`_choice_fn`]    choice from fn
 # @option    --ofb[?`_choice_fn`]   choice from fn + no validation
 # @option    --ofc*[`_choice_fn`]   multi-occurs + choice from fn
 # @option    --ofd*,[`_choice_fn`]  multi-occurs + choice from fn + comma-separated list
 # @option    --oxa~                 capture all remaining args
 options() {
-    _debug "$@";
+    _debug "$@"
 }
 
 # @cmd All kind of flags
@@ -32,7 +34,7 @@ options() {
 # @flag     --fd*        multi-occurs
 # @flag  -e --fe*        short + multi-occurs
 flags() {
-    _debug "$@";
+    _debug "$@"
 }
 
 # @cmd Flags or options with single hyphen
@@ -45,7 +47,7 @@ flags() {
 # @option  -oca[a|b]
 # @option  -ofa[`_choice_fn`]
 options-one-hyphen() {
-    _debug "$@";
+    _debug "$@"
 }
 
 # @cmd Value notation modifier
@@ -53,7 +55,7 @@ options-one-hyphen() {
 # @option --ob <VALUE+>           multi values, one or more
 # @option --oc <VALUE?>           zero or one
 options-notation-modifier() {
-    _debug "$@";
+    _debug "$@"
 }
 
 # @cmd All kind of options
@@ -78,7 +80,7 @@ options-notation-modifier() {
 # @option     +ofd*,[`_choice_fn`]  multi-occurs + choice from fn + comma-separated list
 # @option     +oxa~                 capture all remaining args
 options-plus() {
-    _debug "$@";
+    _debug "$@"
 }
 
 # @cmd All kind of flags
@@ -88,7 +90,7 @@ options-plus() {
 # @flag      +fd*        multi-occurs
 # @flag  +e  +fe*        short + multi-occurs
 flags-plus() {
-    _debug "$@";
+    _debug "$@"
 }
 
 # @cmd Mixed `-` and `+` options
@@ -96,16 +98,21 @@ flags-plus() {
 # @option -b +b
 # @option +c --c
 options-mixed() {
-    _debug "$@";
+    _debug "$@"
 }
 
 # @cmd Prefixed option
 # @option -X-*[`_choice_fn`]       prefixied + multi-occurs + choice from fn
 # @option +X-*[`_choice_fn`]       prefixied + multi-occurs + choice from fn
 options-prefixed() {
-    _debug "$@";
+    _debug "$@"
 }
 
+# @cmd Prefixed option
+# @option -f --follow:[a|b]       assigned + choice
+options-assigned() {
+    _debug "$@"
+}
 
 # @cmd
 # @flag   -a
@@ -125,7 +132,7 @@ options-prefixed() {
 # @option    --ce*[`_choice_fn`]
 # @option -s -soa
 test1() {
-    _debug "$@";
+    _debug "$@"
 }
 
 # @cmd
@@ -136,7 +143,7 @@ test1() {
 # @option    --ocb+[`_choice_fn`]
 # @option    --occ+,[`_choice_fn`]
 test2() {
-    _debug "$@";
+    _debug "$@"
 }
 
 # @cmd
@@ -144,7 +151,7 @@ test2() {
 # @option    --of=`_default_fn`
 # @option    --cb[=x|y|z]
 test3() {
-    _debug "$@";
+    _debug "$@"
 }
 
 _debug() {
