@@ -3,7 +3,7 @@ mod parallel;
 
 use anyhow::{anyhow, bail, Context, Result};
 use argc::{
-    utils::{escape_shell_words, get_current_dir, get_shell_path, termwidth},
+    utils::{escape_shell_words, get_current_dir, get_shell_path, is_true_value, termwidth},
     Shell,
 };
 use base64::{engine::general_purpose, Engine as _};
@@ -214,7 +214,7 @@ fn run_compgen(mut args: Vec<String>) -> Option<()> {
         }
     }
     let no_color = std::env::var("NO_COLOR")
-        .map(|v| v == "true" || v == "1")
+        .map(|v| is_true_value(&v))
         .unwrap_or_default();
     let output = if &args[4] == "argc" && (args[3].is_empty() || args[5].starts_with("--argc")) {
         let cmd_args = &args[4..];
@@ -267,13 +267,13 @@ fn get_argc_help() -> String {
 
 USAGE:
     argc --argc-eval <SCRIPT> [ARGS]...             Use `eval "$(argc --argc-eval "$0" "$@")"`
-    argc --argc-create [TASKS]...                   Create a boilerplate argcfile
+    argc --argc-create [RECIPES]...                 Create a boilerplate argcfile
     argc --argc-build <SCRIPT> [OUTPATH]            Generate bashscript without argc dependency
     argc --argc-mangen <SCRIPT> <OUTDIR>            Generate man pages
     argc --argc-completions <SHELL> [CMDS]...       Generate shell completion scripts
     argc --argc-compgen <SHELL> <SCRIPT> <ARGS>...  Generate completion candidates
     argc --argc-export <SCRIPT>                     Export command line definitions as json
-    argc --argc-parallel <SCRIPT> <ARGS>...         Execute argc functions in parallel
+    argc --argc-parallel <SCRIPT> <ARGS>...         Run functions in parallel
     argc --argc-script-path                         Print current argcfile path
     argc --argc-shell-path                          Print current shell path
     argc --argc-help                                Print help information
@@ -349,7 +349,7 @@ fn generate_boilerplate(args: &[String]) -> String {
                 r#"
 # @cmd
 {cmd}() {{
-    echo To implement command: {cmd}
+    echo TODO {cmd}
 }}
 "#
             )
@@ -362,6 +362,7 @@ fn generate_boilerplate(args: &[String]) -> String {
 
 set -e
 {tasks}
+# See more details at https://github.com/sigoden/argc
 eval "$(argc --argc-eval "$0" "$@")"
 "#
     )
