@@ -1,12 +1,12 @@
 use crate::{
     command::Command,
-    param::{FlagOptionParam, Param},
-    utils::{escape_shell_words, expand_dotenv, META_DOTENV},
+    param::{FlagOptionParam, Param, PositionalParam},
+    utils::{escape_shell_words, expand_dotenv, ARGC_REQUIRE_TOOLS},
     ChoiceValue, DefaultValue,
 };
 use anyhow::Result;
 
-const UTIL_FNS: [(&str, &str); 5] = [
+const UTIL_FNS: [(&str, &str); 6] = [
     (
         "_argc_take_args",
         r#"
@@ -14,6 +14,9 @@ _argc_take_args() {
     _argc_take_args_values=()
     _argc_take_args_len=0
     local param="$1" min="$2" max="$3" signs="$4" delimiter="$5"
+    if [[ "$min" -eq 0 ]] && [[ "$max" -eq 0 ]]; then
+        return
+    fi
     local _argc_take_index=$((_argc_index + 1)) _argc_take_value
     if [[ "$_argc_item" == *=* ]]; then
         _argc_take_args_values=("${_argc_item##*=}")
@@ -157,6 +160,22 @@ _argc_validate_choices() {
         fi
     done
 }
+"#,
+    ),
+    (
+        "_argc_check_bool",
+        r#"
+_argc_check_bool() {
+    local env_name="$1" param_name=$2
+    local env_value="${!env_name}"
+    if [[ "$env_value" == "true" ]] || [[ "$env_value" == "1" ]]; then
+        return 0
+    elif [[ "$env_value" == "false" ]] || [[ "$env_value" == "0" ]]; then
+        return 1
+    else
+        _argc_die "error: environment variable '$env_name' has invalid value for param '$param_name'"
+    fi
+}
 "#,
     ),
 ];
@@ -193,7 +212,7 @@ pub fn build(source: &str, root_name: &str) -> Result<String> {
 
 fn build_root(cmd: &Command) -> String {
     let command = build_command(cmd);
-    let dotenv = if let Some(value) = cmd.get_metadata(META_DOTENV) {
+    let dotenv = if let Some(value) = cmd.dotenv() {
         format!("\n    {}", expand_dotenv(value))
     } else {
         String::new()
@@ -211,6 +230,13 @@ fn build_root(cmd: &Command) -> String {
             util_fns.push_str(util_fn);
         }
     }
+    let require_tools = if command.contains("_argc_tools") {
+        util_fns.push_str(&format!("\n{ARGC_REQUIRE_TOOLS}\n"));
+        r#"
+    _argc_require_tools "${_argc_tools[@]}""#
+    } else {
+        ""
+    };
 
     format!(
         r#"# ARGC-BUILD {{
@@ -224,8 +250,9 @@ _argc_run() {{
     argc__args=("$(basename "$0" .sh)" "$@")
     argc__positionals=()
     _argc_index=1
-    _argc_len="${{#argc__args[@]}}"
-    _argc_parse{dotenv}{before_hook}
+    _argc_len="${{#argc__args[@]}}"{dotenv}
+    _argc_tools=()
+    _argc_parse{require_tools}{before_hook}
     if [ -n "$argc__fn" ]; then
         $argc__fn "${{argc__positionals[@]}}"{after_hook}
     fi
@@ -247,7 +274,7 @@ _argc_run "$@"
 }
 
 fn build_command(cmd: &Command) -> String {
-    let suffix = if cmd.paths.is_empty() {
+    let suffix = if cmd.is_root() {
         String::new()
     } else {
         format!("_{}", cmd.paths.join("_"))
@@ -295,7 +322,7 @@ _argc_version{suffix}() {{
 
 fn build_parse(cmd: &Command, suffix: &str) -> String {
     let mut parse_help = {
-        let help_flags = cmd.help_flags().join(" | ");
+        let help_flags = cmd.help_flags.join(" | ");
         format!(
             r#"
         {help_flags})
@@ -304,7 +331,7 @@ fn build_parse(cmd: &Command, suffix: &str) -> String {
         )
     };
     let parse_version = if cmd.exist_version() {
-        let version_flags = cmd.version_flags().join(" | ");
+        let version_flags = cmd.version_flags.join(" | ");
         format!(
             r#"
         {version_flags})
@@ -444,8 +471,10 @@ fn build_parse(cmd: &Command, suffix: &str) -> String {
         )
     };
 
+    let flag_option_bind_envs = build_flag_option_bind_envs(cmd);
     let required_flag_options = build_required_flag_options(cmd);
 
+    let require_tools = build_require_tools(cmd);
     let handle = build_handle(cmd, suffix);
 
     if cmd.delegated() {
@@ -476,7 +505,7 @@ _argc_parse{suffix}() {{
         _argc_key="${{_argc_item%%=*}}"
         case "$_argc_key" in{combined_case}
         esac
-    done{required_flag_options}
+    done{flag_option_bind_envs}{required_flag_options}{require_tools}
     if [[ -n "$_argc_action" ]]; then
         $_argc_action
     else{handle}
@@ -488,9 +517,9 @@ _argc_parse{suffix}() {{
 
 fn build_parse_flag_option(param: &FlagOptionParam, signs: &str) -> String {
     let names = param.list_names().join(" | ");
-    let long_name = param.render_long_name();
+    let long_name = param.long_name();
     let var_name = param.var_name();
-    if param.is_flag {
+    if param.is_flag() {
         if param.id() == "help" || param.id() == "version" {
             return String::new();
         }
@@ -518,7 +547,7 @@ fn build_parse_flag_option(param: &FlagOptionParam, signs: &str) -> String {
         )
     } else {
         let signs = if param.terminated() { "" } else { signs };
-        let delimiter = match param.args_delimiter() {
+        let delimiter = match param.delimiter() {
             Some(v) => v.to_string(),
             None => String::new(),
         };
@@ -545,11 +574,35 @@ fn build_parse_flag_option(param: &FlagOptionParam, signs: &str) -> String {
             fi"#
             )
         };
-        let (min, max) = param.args_range();
+        let (min, max) = param.num_args();
+        let code = if param.assigned() {
+            let not_assigned = if min == 1 {
+                format!(
+                    r#"
+                _argc_die "error: incorrect number of values for \`{render_name_notations}\`""#
+                )
+            } else {
+                r#"
+                _argc_take_args_values=()
+                _argc_take_args_len=0"#
+                    .into()
+            };
+            format!(
+                r#"
+            if [[ "$_argc_key" == "$_argc_item" ]]; then{not_assigned}
+            else
+                _argc_take_args "{render_name_notations}" {min} {max} "{signs}" "{delimiter}"
+            fi"#
+            )
+        } else {
+            format!(
+                r#"
+            _argc_take_args "{render_name_notations}" {min} {max} "{signs}" "{delimiter}""#
+            )
+        };
         format!(
             r#"
-        {names})
-            _argc_take_args "{render_name_notations}" {min} {max} "{signs}" "{delimiter}"
+        {names}){code}
             _argc_index=$((_argc_index + _argc_take_args_len + 1)){choice}{variant}
             ;;"#
         )
@@ -595,6 +648,22 @@ fn build_handle(cmd: &Command, suffix: &str) -> String {
     }
 }
 
+fn build_require_tools(cmd: &Command) -> String {
+    if cmd.require_tools.is_empty() {
+        return String::new();
+    }
+    let tools = cmd
+        .require_tools
+        .iter()
+        .map(|v| escape_shell_words(v))
+        .collect::<Vec<_>>()
+        .join(" ");
+    format!(
+        r#"
+    _argc_tools=({tools})"#
+    )
+}
+
 fn build_positionals(cmd: &Command) -> String {
     if cmd.positional_params.is_empty() {
         return String::new();
@@ -614,7 +683,7 @@ fn build_positionals(cmd: &Command) -> String {
             let render_value = param.render_notation();
             let multiple = param.multiple_values();
             let variant = if multiple {
-                match param.args_delimiter() {
+                match param.delimiter() {
                     Some(delimiter) => format!(
                         r#"
             _argc_split_positionals "$values_index" "$values_size" "{delimiter}"
@@ -661,6 +730,9 @@ fn build_positionals(cmd: &Command) -> String {
             } else {
                 String::new()
             };
+
+            let bind_env = build_positional_bind_env(param);
+
             let handle_nonexist = format!("{default}{required}");
             let handle_nonexist = if !handle_nonexist.is_empty() {
                 format!(
@@ -673,7 +745,7 @@ fn build_positionals(cmd: &Command) -> String {
             format!(
                 r#"
         IFS=: read -r values_index values_size <<<"${{_argc_match_positionals_values[{index}]}}"
-        if [[ -n "$values_index" ]]; then{variant}{choice}{handle_nonexist}
+        if [[ -n "$values_index" ]]; then{variant}{choice}{bind_env}{handle_nonexist}
         fi"#
             )
         })
@@ -686,6 +758,85 @@ fn build_positionals(cmd: &Command) -> String {
     )
 }
 
+fn build_flag_option_bind_envs(cmd: &Command) -> String {
+    let mut output = vec![];
+    for param in &cmd.flag_option_params {
+        if let Some(env_name) = param.bind_env() {
+            let var_name = param.var_name();
+            let render_name = param.render_name_notations();
+            let code = if param.is_flag() {
+                format!(
+                    r#"
+    if [[ -z "${var_name}" ]] && [[ -n "${env_name}" ]]; then
+        if _argc_check_bool {env_name} "{render_name}"; then
+            {var_name}=1
+        fi
+    fi"#
+                )
+            } else {
+                let handle_bind_env = build_handle_bind_env(param, &render_name, 2);
+                format!(
+                    r#"
+    if [[ -z "${var_name}" ]] && [[ -n "${env_name}" ]]; then{handle_bind_env}
+    fi"#
+                )
+            };
+            output.push(code);
+        }
+    }
+    output.join("")
+}
+
+fn build_positional_bind_env(param: &PositionalParam) -> String {
+    match param.bind_env() {
+        None => String::new(),
+        Some(env_name) => {
+            let handle_bind_env = build_handle_bind_env(param, &param.render_notation(), 3);
+            format!(
+                r#"
+        elif [[ -n "${env_name}" ]]; then{handle_bind_env}
+            argc__positionals+=("${{_argc_env_values[@]}}")"#
+            )
+        }
+    }
+}
+
+fn build_handle_bind_env<T: Param>(param: &T, render_name: &str, indent_level: usize) -> String {
+    let indent = build_indent(indent_level);
+    let env_name = param.bind_env().unwrap_or_default();
+    let var_name = param.var_name();
+    let split_env = match param.delimiter() {
+        Some(delimiter) => format!(
+            r#"
+{indent}IFS="{delimiter}" read -r -a _argc_env_values <<<"${env_name}""#
+        ),
+        None => format!(
+            r#"
+{indent}_argc_env_values=("${env_name}")"#
+        ),
+    };
+
+    let choice = build_choice(
+        "{_argc_env_values[@]}",
+        &format!(r#"environment variable `{env_name}` that bound to `{render_name}`"#),
+        param.choice(),
+        indent_level,
+    );
+
+    let variant = if param.multiple_values() {
+        format!(
+            r#"
+{indent}{var_name}=("${{_argc_env_values[@]}}")"#
+        )
+    } else {
+        format!(
+            r#"
+{indent}{var_name}="${{_argc_env_values[0]}}""#
+        )
+    };
+    format!(r#"{indent}{split_env}{choice}{variant}"#)
+}
+
 fn build_required_flag_options(cmd: &Command) -> String {
     let required_flag_options: Vec<_> = cmd
         .flag_option_params
@@ -802,8 +953,8 @@ fn build_envs(cmd: &Command) -> String {
         .join("")
 }
 
-fn build_default(var_name: &str, value: Option<&DefaultValue>, indent: usize) -> String {
-    let indent = build_indent(indent);
+fn build_default(var_name: &str, value: Option<&DefaultValue>, indent_level: usize) -> String {
+    let indent = build_indent(indent_level);
     match value {
         Some(value) => match value {
             DefaultValue::Value(value) => {
@@ -851,6 +1002,6 @@ fn build_choice(
     }
 }
 
-fn build_indent(indent: usize) -> String {
-    "    ".repeat(indent)
+fn build_indent(indent_level: usize) -> String {
+    "    ".repeat(indent_level)
 }
