@@ -7,14 +7,13 @@ use self::share_data::ShareData;
 use crate::argc_value::ArgcValue;
 use crate::matcher::Matcher;
 use crate::param::{
-    EnvParam, EnvValue, FlagOptionParam, FlagOptionValue, Param, ParamData, PositionalParam,
-    PositionalValue,
+    EnvParam, EnvValue, FlagOptionParam, FlagOptionValue, Param, PositionalParam, PositionalValue,
 };
 use crate::parser::{parse, parse_symbol, Event, EventData, EventScope, Position};
 use crate::utils::{
-    AFTER_HOOK, BEFORE_HOOK, INTERNAL_SYMBOL, MAIN_NAME, META_COMBINE_SHORTS,
-    META_DEFAULT_SUBCOMMAND, META_DOTENV, META_INHERIT_FLAG_OPTIONS, META_NO_INHERIT_ENV,
-    META_SYMBOL, ROOT_NAME,
+    AFTER_HOOK, BEFORE_HOOK, INTERNAL_SYMBOL, MAIN_NAME, META_AUTHOR, META_COMBINE_SHORTS,
+    META_DEFAULT_SUBCOMMAND, META_DOTENV, META_INHERIT_FLAG_OPTIONS, META_REQUIRE_TOOLS,
+    META_SYMBOL, META_VERSION, ROOT_NAME,
 };
 use crate::Result;
 
@@ -44,8 +43,12 @@ pub(crate) struct Command {
     pub(crate) version: Option<String>,
     pub(crate) names_checker: NamesChecker,
     pub(crate) share: Arc<RefCell<ShareData>>,
+    // (key, value, position)
     pub(crate) metadata: Vec<(String, String, Position)>,
     pub(crate) symbols: IndexMap<char, SymbolParam>,
+    pub(crate) require_tools: IndexSet<String>,
+    pub(crate) help_flags: Vec<&'static str>,
+    pub(crate) version_flags: Vec<&'static str>,
 }
 
 impl Command {
@@ -53,13 +56,11 @@ impl Command {
         let events = parse(source)?;
         let mut root = Command::new_from_events(&events)?;
         root.share.borrow_mut().name = Some(root_name.to_string());
-        root.update_recursively(vec![]);
+        root.update_recursively(vec![], IndexSet::new());
         if root.has_metadata(META_INHERIT_FLAG_OPTIONS) {
             root.inherit_flag_options();
         }
-        if !root.has_metadata(META_NO_INHERIT_ENV) {
-            root.inherit_envs();
-        }
+        root.inherit_envs();
         Ok(root)
     }
 
@@ -96,16 +97,15 @@ impl Command {
 
     pub(crate) fn export(&self) -> CommandValue {
         let mut extra: IndexMap<String, serde_json::Value> = IndexMap::new();
-        if self.paths.is_empty() {
+        let require_tools = self.meta_require_tools();
+        if !require_tools.is_empty() {
+            extra.insert("require_tools".into(), require_tools.into());
+        }
+        if self.is_root() {
             if self.get_metadata(META_COMBINE_SHORTS).is_some() {
                 extra.insert("combine_shorts".into(), true.into());
             }
-            if let Some(dotenv) = self.get_metadata(META_DOTENV) {
-                let dotenv = if dotenv.is_empty() {
-                    ".env".to_string()
-                } else {
-                    dotenv.to_string()
-                };
+            if let Some(dotenv) = self.dotenv() {
                 extra.insert("dotenv".into(), dotenv.into());
             }
             let (before_hook, after_hook) = self.exist_hooks();
@@ -118,6 +118,7 @@ impl Command {
         } else if let Some((idx, _)) = &self.default_subcommand {
             extra.insert("default_subcommand".into(), (*idx).into());
         }
+        extra.insert("command_fn".into(), self.command_fn.clone().into());
         let flag_options = self.all_flag_options().iter().map(|v| v.export()).collect();
         CommandValue {
             name: self.cmd_name(),
@@ -129,8 +130,6 @@ impl Command {
             positionals: self.positional_params.iter().map(|v| v.export()).collect(),
             envs: self.env_params.iter().map(|v| v.export()).collect(),
             subcommands: self.subcommands.iter().map(|v| v.export()).collect(),
-            paths: self.paths.clone(),
-            command_fn: self.command_fn.clone(),
             extra,
         }
     }
@@ -155,12 +154,29 @@ impl Command {
                 }
                 EventData::Meta(key, value) => {
                     let cmd = Self::get_cmd(&mut root_cmd, "@meta", position)?;
-                    if key == META_SYMBOL {
-                        let (ch, name, choice_fn) = parse_symbol(&value).ok_or_else(|| {
-                            anyhow!("@meta(line {}) invalid symbol value", position)
-                        })?;
-                        cmd.symbols
-                            .insert(ch, (name.to_string(), choice_fn.map(|v| v.to_string())));
+                    match key.as_str() {
+                        META_SYMBOL => {
+                            let (ch, name, choice_fn) = parse_symbol(&value).ok_or_else(|| {
+                                anyhow!("@meta(line {}) invalid symbol value", position)
+                            })?;
+                            cmd.symbols
+                                .insert(ch, (name.to_string(), choice_fn.map(|v| v.to_string())));
+                        }
+                        META_VERSION => {
+                            if value.is_empty() {
+                                bail!("@meta(line {}) invalid version value", position)
+                            } else {
+                                cmd.version = Some(value.clone());
+                            }
+                        }
+                        META_AUTHOR => {
+                            if value.is_empty() {
+                                bail!("@meta(line {}) invalid version value", position)
+                            } else {
+                                cmd.author = Some(value.clone());
+                            }
+                        }
+                        _ => {}
                     }
                     cmd.metadata.push((key, value, position));
                 }
@@ -320,13 +336,25 @@ impl Command {
             .map(|(_, v, _)| v.as_str())
     }
 
+    pub(crate) fn meta_require_tools(&self) -> Vec<String> {
+        let raw_require_tools = self.get_metadata(META_REQUIRE_TOOLS).unwrap_or_default();
+        if raw_require_tools.is_empty() {
+            vec![]
+        } else {
+            raw_require_tools
+                .split(',')
+                .map(|v| v.to_string())
+                .collect()
+        }
+    }
+
     pub(crate) fn flag_option_signs(&self) -> String {
         let mut signs: IndexSet<char> = ['-'].into();
         for param in &self.flag_option_params {
-            if let Some(short) = &param.short {
+            if let Some(short) = param.short() {
                 signs.extend(short.chars().take(1))
             }
-            signs.extend(param.long_prefix.chars().take(1))
+            signs.extend(param.long_prefix().chars().take(1))
         }
         signs.into_iter().collect()
     }
@@ -337,6 +365,10 @@ impl Command {
             .unwrap_or_else(|| self.share.borrow().name())
     }
 
+    pub(crate) fn is_root(&self) -> bool {
+        self.paths.is_empty()
+    }
+
     pub(crate) fn cmd_paths(&self) -> Vec<String> {
         let root_name = self.share.borrow().name();
         let mut paths = self.paths.clone();
@@ -382,130 +414,6 @@ impl Command {
         )
     }
 
-    pub(crate) fn render_usage(&self) -> String {
-        let mut output = vec!["USAGE:".to_string()];
-        output.extend(self.cmd_paths());
-        let required_options: Vec<String> = self
-            .flag_option_params
-            .iter()
-            .filter(|v| v.required())
-            .map(|v| v.render_name_notations())
-            .collect();
-        if self.flag_option_params.len() != required_options.len() {
-            output.push("[OPTIONS]".to_string());
-        }
-        output.extend(required_options);
-        if !self.subcommands.is_empty() {
-            output.push("<COMMAND>".to_string());
-        } else {
-            output.extend(self.positional_params.iter().map(|v| v.render_notation()));
-        }
-        output.join(" ")
-    }
-
-    pub(crate) fn render_positionals(&self, term_width: Option<usize>) -> Vec<String> {
-        let mut output = vec![];
-        if self.positional_params.is_empty() {
-            return output;
-        }
-        let mut value_size = 0;
-        let list: Vec<_> = self
-            .positional_params
-            .iter()
-            .map(|param| {
-                let value = param.render_notation();
-                value_size = value_size.max(value.len());
-                (value, param.render_describe())
-            })
-            .collect();
-        value_size += 2;
-        output.push("ARGS:".to_string());
-        render_list(&mut output, list, value_size, term_width);
-        output
-    }
-
-    pub(crate) fn render_flag_options(&self, term_width: Option<usize>) -> Vec<String> {
-        let mut output = vec![];
-        if self.flag_option_params.is_empty() {
-            return output;
-        }
-        let mut value_size = 0;
-        let list: Vec<_> = self
-            .all_flag_options()
-            .into_iter()
-            .map(|param| {
-                let value = param.render_body();
-                let describe = param.render_describe();
-                value_size = value_size.max(value.len());
-                (value, describe)
-            })
-            .collect();
-        value_size += 2;
-        output.push("OPTIONS:".to_string());
-        render_list(&mut output, list, value_size, term_width);
-        output
-    }
-
-    pub(crate) fn render_subcommands(&self, term_width: Option<usize>) -> Vec<String> {
-        let mut output = vec![];
-        if self.subcommands.is_empty() {
-            return output;
-        }
-        let mut value_size = 0;
-        let list: Vec<_> = self
-            .subcommands
-            .iter()
-            .map(|subcmd| {
-                let value = subcmd.cmd_name();
-                value_size = value_size.max(value.len());
-                (value, subcmd.render_subcommand_describe())
-            })
-            .collect();
-        value_size += 2;
-        output.push("COMMANDS:".to_string());
-        render_list(&mut output, list, value_size, term_width);
-        output
-    }
-
-    pub(crate) fn render_subcommand_describe(&self) -> String {
-        let mut output = self.describe_oneline().to_string();
-        if let Some((aliases, _)) = &self.aliases {
-            if !output.is_empty() {
-                output.push(' ')
-            }
-            output.push_str(&format!("[aliases: {}]", aliases.join(", ")));
-        }
-        if self.has_metadata(META_DEFAULT_SUBCOMMAND) {
-            if !output.is_empty() {
-                output.push(' ')
-            }
-            output.push_str("[default]");
-        }
-        output
-    }
-
-    pub(crate) fn render_envs(&self, term_width: Option<usize>) -> Vec<String> {
-        let mut output = vec![];
-        if self.env_params.is_empty() {
-            return output;
-        }
-        let mut value_size = 0;
-        let list: Vec<_> = self
-            .env_params
-            .iter()
-            .map(|param| {
-                let value = param.render_body();
-                value_size = value_size.max(value.len());
-                (value, param.render_describe())
-            })
-            .collect();
-        value_size += 2;
-        output.push("ENVIRONMENTS:".to_string());
-        render_list(&mut output, list, value_size, term_width);
-        output.push("".to_string());
-        output
-    }
-
     pub(crate) fn describe_oneline(&self) -> &str {
         match self.describe.split_once('\n') {
             Some((v, _)) => v,
@@ -573,35 +481,7 @@ impl Command {
     }
 
     pub(crate) fn exist_version(&self) -> bool {
-        self.version.is_some() || self.paths.is_empty()
-    }
-
-    pub(crate) fn help_flags(&self) -> Vec<&'static str> {
-        let mut output = vec!["--help", "-help"];
-        let short = match self.find_flag_option("-h") {
-            Some(param) => param.id() == "help",
-            None => true,
-        };
-        if short {
-            output.push("-h");
-        }
-        output
-    }
-
-    pub(crate) fn version_flags(&self) -> Vec<&'static str> {
-        let mut output = vec![];
-        if self.exist_version() {
-            output.push("--version");
-            output.push("-version");
-            let short = match self.find_flag_option("-V") {
-                Some(param) => param.id() == "version",
-                None => true,
-            };
-            if short {
-                output.push("-V");
-            }
-        }
-        output
+        self.version.is_some() || self.is_root()
     }
 
     pub(crate) fn delegated(&self) -> bool {
@@ -611,7 +491,13 @@ impl Command {
             && self.positional_params[0].terminated()
     }
 
-    fn update_recursively(&mut self, paths: Vec<String>) {
+    pub(crate) fn dotenv(&self) -> Option<&str> {
+        let dotenv = self.get_metadata(META_DOTENV)?;
+        let dotenv = if dotenv.is_empty() { ".env" } else { dotenv };
+        Some(dotenv)
+    }
+
+    fn update_recursively(&mut self, paths: Vec<String>, mut require_tools: IndexSet<String>) {
         self.paths = paths.clone();
 
         // update command_fn
@@ -630,11 +516,38 @@ impl Command {
             }
         }
 
+        self.help_flags = {
+            let mut flags = vec!["--help", "-help"];
+            let short = match self.find_flag_option("-h") {
+                Some(param) => param.id() == "help",
+                None => true,
+            };
+            if short {
+                flags.push("-h");
+            }
+            flags
+        };
+        self.version_flags = {
+            let mut flags = vec![];
+            if self.exist_version() {
+                flags.push("--version");
+                flags.push("-version");
+                let short = match self.find_flag_option("-V") {
+                    Some(param) => param.id() == "version",
+                    None => true,
+                };
+                if short {
+                    flags.push("-V");
+                }
+            }
+            flags
+        };
+
         // update derived_flag_option_params
         let mut describe = false;
         let mut single = false;
         for param in self.flag_option_params.iter() {
-            if param.long_prefix.len() == 1 {
+            if param.long_prefix().len() == 1 {
                 single = true;
             }
             if !param.describe().is_empty() {
@@ -651,11 +564,15 @@ impl Command {
             .flatten(),
         );
 
+        // update require_tools
+        require_tools.extend(self.meta_require_tools());
+        self.require_tools = require_tools;
+
         // update recursively
         for subcmd in self.subcommands.iter_mut() {
             let mut parents = paths.clone();
             parents.push(subcmd.name.clone().unwrap_or_default());
-            subcmd.update_recursively(parents);
+            subcmd.update_recursively(parents, self.require_tools.clone());
         }
     }
 
@@ -665,7 +582,7 @@ impl Command {
             for flag_option in &self.flag_option_params {
                 if subcmd.find_flag_option(flag_option.id()).is_none() {
                     let mut flag_option = flag_option.clone();
-                    flag_option.inherit = true;
+                    flag_option.set_inherit();
                     inherited_flag_options.push(flag_option);
                 }
             }
@@ -684,7 +601,7 @@ impl Command {
             for env_param in &self.env_params {
                 if subcmd.find_env(env_param.id()).is_none() {
                     let mut env_param = env_param.clone();
-                    env_param.inherit = true;
+                    env_param.set_inherit();
                     inherited_envs.push(env_param);
                 }
             }
@@ -736,13 +653,10 @@ impl Command {
         } else {
             None
         };
-        Some(FlagOptionParam::new(
-            ParamData::new("help"),
-            describe,
-            true,
+        Some(FlagOptionParam::create_help_flag(
             short,
             long_prefix,
-            &[],
+            describe,
         ))
     }
 
@@ -759,15 +673,157 @@ impl Command {
         } else {
             None
         };
-        Some(FlagOptionParam::new(
-            ParamData::new("version"),
-            describe,
-            true,
+        Some(FlagOptionParam::create_version_flag(
             short,
             long_prefix,
-            &[],
+            describe,
         ))
     }
+
+    fn render_usage(&self) -> String {
+        let mut output = vec!["USAGE:".to_string()];
+        output.extend(self.cmd_paths());
+        let required_options: Vec<String> = self
+            .flag_option_params
+            .iter()
+            .filter(|v| v.required())
+            .map(|v| v.render_name_notations())
+            .collect();
+        if self.flag_option_params.len() != required_options.len() {
+            output.push("[OPTIONS]".to_string());
+        }
+        output.extend(required_options);
+        if !self.subcommands.is_empty() {
+            output.push("<COMMAND>".to_string());
+        } else {
+            output.extend(self.positional_params.iter().map(|v| v.render_notation()));
+        }
+        output.join(" ")
+    }
+
+    fn render_flag_options(&self, term_width: Option<usize>) -> Vec<String> {
+        let mut output = vec![];
+        let default_subcmd = self.find_default_subcommand();
+        if self.flag_option_params.is_empty()
+            && default_subcmd
+                .map(|subcmd| subcmd.flag_option_params.is_empty())
+                .unwrap_or(true)
+        {
+            return output;
+        }
+
+        let params = match default_subcmd {
+            Some(subcmd) => [self.all_flag_options(), subcmd.all_flag_options()].concat(),
+            None => self.all_flag_options(),
+        };
+
+        let mut value_size = 0;
+        let list: IndexMap<String, String> = params
+            .into_iter()
+            .map(|param| {
+                let value = param.render_body();
+                let describe = param.render_describe();
+                value_size = value_size.max(value.len());
+                (value, describe)
+            })
+            .collect();
+        value_size += 2;
+        output.push("OPTIONS:".to_string());
+        render_list(
+            &mut output,
+            list.into_iter().collect(),
+            value_size,
+            term_width,
+        );
+        output
+    }
+
+    fn render_positionals(&self, term_width: Option<usize>) -> Vec<String> {
+        let mut output = vec![];
+        let params = match self.find_default_subcommand() {
+            Some(subcmd) => &subcmd.positional_params,
+            None => &self.positional_params,
+        };
+        if params.is_empty() {
+            return output;
+        }
+        let mut value_size = 0;
+        let list: Vec<_> = params
+            .iter()
+            .map(|param| {
+                let value = param.render_notation();
+                value_size = value_size.max(value.len());
+                (value, param.render_describe())
+            })
+            .collect();
+        value_size += 2;
+        output.push("ARGS:".to_string());
+        render_list(&mut output, list, value_size, term_width);
+        output
+    }
+
+    fn render_envs(&self, term_width: Option<usize>) -> Vec<String> {
+        let mut output = vec![];
+        let params = match self.find_default_subcommand() {
+            Some(subcmd) => &subcmd.env_params,
+            None => &self.env_params,
+        };
+        if params.is_empty() {
+            return output;
+        }
+        let mut value_size = 0;
+        let list: Vec<_> = params
+            .iter()
+            .map(|param| {
+                let value = param.render_body();
+                value_size = value_size.max(value.len());
+                (value, param.render_describe())
+            })
+            .collect();
+        value_size += 2;
+        output.push("ENVIRONMENTS:".to_string());
+        render_list(&mut output, list, value_size, term_width);
+        output.push("".to_string());
+        output
+    }
+
+    fn render_subcommands(&self, term_width: Option<usize>) -> Vec<String> {
+        let mut output = vec![];
+        if self.subcommands.is_empty() {
+            return output;
+        }
+        let mut value_size = 0;
+        let list: Vec<_> = self
+            .subcommands
+            .iter()
+            .map(|subcmd| {
+                let value = subcmd.cmd_name();
+                value_size = value_size.max(value.len());
+                (value, subcmd.render_subcommand_describe())
+            })
+            .collect();
+        value_size += 2;
+        output.push("COMMANDS:".to_string());
+        render_list(&mut output, list, value_size, term_width);
+        output
+    }
+
+    fn render_subcommand_describe(&self) -> String {
+        let mut output = self.describe_oneline().to_string();
+        if let Some((aliases, _)) = &self.aliases {
+            if !output.is_empty() {
+                output.push(' ')
+            }
+            output.push_str(&format!("[aliases: {}]", aliases.join(", ")));
+        }
+        if self.has_metadata(META_DEFAULT_SUBCOMMAND) {
+            if !output.is_empty() {
+                output.push(' ')
+            }
+            output.push_str("[default]");
+        }
+        output
+    }
 }
 
 #[derive(Debug, Serialize)]
@@ -781,8 +837,6 @@ pub struct CommandValue {
     pub positionals: Vec<PositionalValue>,
     pub envs: Vec<EnvValue>,
     pub subcommands: Vec<CommandValue>,
-    pub paths: Vec<String>,
-    pub command_fn: Option<String>,
     #[serde(flatten)]
     pub extra: IndexMap<String, serde_json::Value>,
 }
