@@ -1,16 +1,16 @@
 #![allow(clippy::too_many_arguments)]
 
-use std::{
-    collections::{HashMap, HashSet},
-    env,
-};
+use std::collections::{HashMap, HashSet};
 
 use crate::{
     argc_value::ArgcValue,
     command::{Command, SymbolParam},
     compgen::CompColor,
     param::{ChoiceValue, FlagOptionParam, Param, ParamData, PositionalParam},
-    utils::{argc_var_name, run_param_fns, META_COMBINE_SHORTS, META_DOTENV},
+    utils::{
+        all_envs, argc_var_name, get_os, is_true_value, load_dotenv, run_param_fns,
+        META_COMBINE_SHORTS,
+    },
     Shell,
 };
 
@@ -28,7 +28,7 @@ pub(crate) struct Matcher<'a, 'b> {
     arg_comp: ArgComp,
     choice_fns: HashSet<&'a str>,
     script_path: Option<String>,
-    envs: HashMap<&'a str, String>,
+    envs: HashMap<String, String>,
     term_width: Option<usize>,
     split_last_arg_at: Option<usize>,
     comp_option: Option<&'a str>,
@@ -64,6 +64,7 @@ pub(crate) enum MatchError {
     MissingRequiredEnvironments(Vec<String>),
     NotMultipleArgument(usize, String),
     InvalidValue(usize, String, String, Vec<String>),
+    InvalidBindEnvironment(usize, String, String, String, Vec<String>),
     InvalidEnvironment(usize, String, String, Vec<String>),
     MismatchValues(usize, String),
     NoFlagValue(usize, String),
@@ -119,8 +120,8 @@ impl<'a, 'b> Matcher<'a, 'b> {
                     }
                 } else if is_rest_args_positional
                     || (cmd.is_empty_flags_options_subcommands()
-                        && !cmd.help_flags().contains(&arg)
-                        && !cmd.version_flags().contains(&arg))
+                        && !cmd.help_flags.contains(&arg)
+                        && !cmd.version_flags.contains(&arg))
                 {
                     add_positional_arg(
                         &mut positional_args,
@@ -145,6 +146,23 @@ impl<'a, 'b> Matcher<'a, 'b> {
                             flag_option_args[cmd_level].push((k, values, Some(param.id())));
                             comp_option = Some(param.id());
                         } else {
+                            if positional_args.is_empty()
+                                && !cmd.help_flags.contains(&k)
+                                && !cmd.version_flags.contains(&k)
+                            {
+                                if let Some(subcmd) = cmd.find_default_subcommand() {
+                                    match_command(
+                                        &mut cmds,
+                                        &mut cmd_level,
+                                        &mut cmd_arg_indexes,
+                                        &mut flag_option_args,
+                                        subcmd,
+                                        arg_index,
+                                        &mut is_rest_args_positional,
+                                    );
+                                    continue;
+                                }
+                            }
                             flag_option_args[cmd_level].push((k, vec![v], None));
                         }
                     } else if let Some(param) = cmd.find_flag_option(arg) {
@@ -227,6 +245,23 @@ impl<'a, 'b> Matcher<'a, 'b> {
                             split_last_arg_at = Some(1);
                         }
                     } else {
+                        if positional_args.is_empty()
+                            && !cmd.help_flags.contains(&arg)
+                            && !cmd.version_flags.contains(&arg)
+                        {
+                            if let Some(subcmd) = cmd.find_default_subcommand() {
+                                match_command(
+                                    &mut cmds,
+                                    &mut cmd_level,
+                                    &mut cmd_arg_indexes,
+                                    &mut flag_option_args,
+                                    subcmd,
+                                    arg_index,
+                                    &mut is_rest_args_positional,
+                                );
+                                continue;
+                            }
+                        }
                         flag_option_args[cmd_level].push((arg, vec![], None));
                     }
                 } else if let Some(subcmd) =
@@ -286,18 +321,34 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 arg_comp = ArgComp::CommandOrPositional;
             }
 
-            let last_cmd = *cmds.last().unwrap();
+            let last_cmd = cmds[cmd_level];
+            if positional_args.is_empty() && flag_option_args[cmd_level].is_empty() {
+                if let Some(subcmd) = last_cmd.find_default_subcommand() {
+                    if subcmd.command_fn.is_some() {
+                        match_command(
+                            &mut cmds,
+                            &mut cmd_level,
+                            &mut cmd_arg_indexes,
+                            &mut flag_option_args,
+                            subcmd,
+                            arg_index - 1,
+                            &mut is_rest_args_positional,
+                        );
+                    }
+                }
+            }
+
             for param in &last_cmd.positional_params {
                 add_param_choice_fn(&mut choice_fns, param)
             }
         }
 
-        let mut envs = HashMap::new();
         let last_cmd = *cmds.last().unwrap();
+
+        let mut envs = all_envs();
+        envs.extend(root_cmd.dotenv().and_then(load_dotenv).unwrap_or_default());
+
         for param in &last_cmd.env_params {
-            if let Ok(value) = std::env::var(param.id()) {
-                envs.insert(param.id(), value);
-            }
             add_param_choice_fn(&mut choice_fns, param)
         }
 
@@ -328,16 +379,22 @@ impl<'a, 'b> Matcher<'a, 'b> {
     }
 
     pub(crate) fn to_arg_values(&self) -> Vec<ArgcValue> {
-        if let Some(err) = self.validate() {
+        let bind_envs = self.build_bind_envs();
+        if let Some(err) = self.validate(&bind_envs) {
             return vec![ArgcValue::Error(self.stringify_match_error(&err))];
         }
         let last_cmd = self.last_cmd();
-        let mut output = self.to_arg_values_base();
+        let mut output = self.to_arg_values_base(&bind_envs);
         if last_cmd.positional_params.is_empty() && !self.positional_args.is_empty() {
             output.push(ArgcValue::ExtraPositionalMultiple(
                 self.positional_args.iter().map(|v| v.to_string()).collect(),
             ));
         }
+        if !last_cmd.require_tools.is_empty() {
+            output.push(ArgcValue::RequireTools(
+                last_cmd.require_tools.iter().cloned().collect(),
+            ));
+        }
         if let Some(command_fn) = &last_cmd.command_fn {
             output.push(ArgcValue::CommandFn(command_fn.clone()));
         }
@@ -345,7 +402,8 @@ impl<'a, 'b> Matcher<'a, 'b> {
     }
 
     pub(crate) fn to_arg_values_for_param_fn(&self) -> Vec<ArgcValue> {
-        let mut output: Vec<ArgcValue> = self.to_arg_values_base();
+        let bind_envs = self.build_bind_envs();
+        let mut output: Vec<ArgcValue> = self.to_arg_values_base(&bind_envs);
         let cmds_len = self.cmds.len();
         let level = cmds_len - 1;
         let last_cmd = self.cmds[level];
@@ -469,19 +527,18 @@ impl<'a, 'b> Matcher<'a, 'b> {
         self.split_last_arg_at
     }
 
-    fn to_arg_values_base(&self) -> Vec<ArgcValue> {
+    fn to_arg_values_base<'x>(&'x self, bind_envs: &BindEnvs<'a, 'x>) -> Vec<ArgcValue> {
         let mut output = vec![];
         let root_cmd = self.cmds[0];
         let cmds_len = self.cmds.len();
-        let level = cmds_len - 1;
-        let last_cmd = self.cmds[level];
+        let last_cmd = self.last_cmd();
 
-        if let Some(value) = root_cmd.get_metadata(META_DOTENV) {
+        if let Some(value) = root_cmd.dotenv() {
             output.push(ArgcValue::Dotenv(value.to_string()))
         }
 
         for param in &last_cmd.env_params {
-            if !self.envs.contains_key(&param.id()) {
+            if !self.envs.contains_key(param.id()) {
                 if let Some(value) = param.get_env_value() {
                     output.push(value);
                 }
@@ -498,10 +555,31 @@ impl<'a, 'b> Matcher<'a, 'b> {
         }
 
         for level in 0..cmds_len {
-            let args = &self.flag_option_args[level];
+            let args = self.flag_option_args[level].as_slice();
             let cmd = self.cmds[level];
             for param in cmd.flag_option_params.iter() {
-                if let Some(value) = param.to_argc_value(args) {
+                let mut args: Vec<_> = args
+                    .iter()
+                    .filter_map(|(name, value, param_name)| {
+                        if param_name == &Some(param.id()) {
+                            Some((*name, value.as_slice()))
+                        } else {
+                            None
+                        }
+                    })
+                    .collect();
+                if args.is_empty() {
+                    if let Some(env_values) = bind_envs.flag_options[level].get(param.id()) {
+                        if param.is_flag() {
+                            if is_true_value(env_values[0]) {
+                                args = vec![("", env_values.as_slice())];
+                            }
+                        } else {
+                            args = vec![("", env_values.as_slice())];
+                        }
+                    }
+                }
+                if let Some(value) = param.to_argc_value(&args) {
                     output.push(value);
                 }
             }
@@ -509,10 +587,15 @@ impl<'a, 'b> Matcher<'a, 'b> {
 
         let positional_values = self.match_positionals();
         for (i, param) in last_cmd.positional_params.iter().enumerate() {
-            let values = positional_values
+            let mut values = positional_values
                 .get(i)
                 .map(|v| v.as_slice())
                 .unwrap_or_default();
+            if values.is_empty() {
+                if let Some(env_values) = bind_envs.positionals.get(param.id()) {
+                    values = env_values.as_slice()
+                }
+            }
             if let Some(value) = param.to_argc_value(values) {
                 output.push(value);
             }
@@ -521,20 +604,49 @@ impl<'a, 'b> Matcher<'a, 'b> {
         output
     }
 
-    fn validate(&self) -> Option<MatchError> {
+    fn build_bind_envs<'x: 'a>(&'x self) -> BindEnvs<'a, 'x> {
         let cmds_len = self.cmds.len();
-        let choices_fn_values = self.run_choices_fns().unwrap_or_default();
+        let last_cmd = self.last_cmd();
+        let mut bind_envs = BindEnvs::new(cmds_len);
 
+        for level in 0..cmds_len {
+            let cmd = self.cmds[level];
+            for param in cmd.flag_option_params.iter() {
+                if let Some(env_value) = param.bind_env().and_then(|v| self.envs.get(&v)) {
+                    let values = delimit_arg_values(param, &[env_value]);
+                    bind_envs.flag_options[level].insert(param.id(), values);
+                    add_param_choice_fn(&mut bind_envs.choice_fns, param);
+                }
+            }
+        }
+
+        for param in last_cmd.positional_params.iter() {
+            if let Some(env_value) = param.bind_env().and_then(|v| self.envs.get(&v)) {
+                let values = delimit_arg_values(param, &[env_value]);
+                bind_envs.positionals.insert(param.id(), values);
+                add_param_choice_fn(&mut bind_envs.choice_fns, param);
+            }
+        }
+        bind_envs
+    }
+
+    fn validate<'x>(&'x self, bind_envs: &BindEnvs<'a, 'x>) -> Option<MatchError> {
+        let cmds_len = self.cmds.len();
+        let choices_fn_values = self.run_choices_fns(bind_envs).unwrap_or_default();
         for level in 0..cmds_len {
             let flag_option_args = &self.flag_option_args[level];
             let cmd = self.cmds[level];
+            let flag_option_bind_envs = &bind_envs.flag_options[level];
             let mut flag_option_map = IndexMap::new();
             let mut missing_flag_options: IndexSet<&str> = cmd
                 .flag_option_params
                 .iter()
-                .filter(|v| v.required())
+                .filter(|v| v.required() && !flag_option_bind_envs.contains_key(v.id()))
                 .map(|v| v.id())
                 .collect();
+
+            let mut check_flag_option_bind_envs: IndexSet<&str> =
+                flag_option_bind_envs.keys().copied().collect();
             for (i, (key, _, name)) in flag_option_args.iter().enumerate() {
                 match (*key, name) {
                     ("--help", _) | ("-help", _) | ("-h", None) | (_, Some("help")) => {
@@ -550,6 +662,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 match *name {
                     Some(name) => {
                         missing_flag_options.swap_remove(name);
+                        check_flag_option_bind_envs.swap_remove(name);
                         flag_option_map.entry(name).or_insert(vec![]).push(i);
                     }
                     None => return Some(MatchError::UnknownArgument(level, key.to_string())),
@@ -561,18 +674,17 @@ impl<'a, 'b> Matcher<'a, 'b> {
                         .iter()
                         .map(|v| flag_option_args[*v].1.as_slice())
                         .collect();
-                    let (min, _) = param.args_range();
+                    let (min, _) = param.num_args();
                     for values in values_list.iter() {
                         if param.is_flag() && !values.is_empty() {
-                            return Some(MatchError::NoFlagValue(level, param.render_long_name()));
+                            return Some(MatchError::NoFlagValue(level, param.long_name()));
                         } else if values.len() < min {
                             return Some(MatchError::MismatchValues(
                                 level,
                                 param.render_name_notations(),
                             ));
                         }
-                        if let Some(choices) =
-                            get_param_choice(&param.data.choice, &choices_fn_values)
+                        if let Some(choices) = get_param_choice(param.choice(), &choices_fn_values)
                         {
                             for value in values.iter() {
                                 if !choices.contains(&value.to_string()) {
@@ -587,13 +699,50 @@ impl<'a, 'b> Matcher<'a, 'b> {
                         }
                     }
                     if !param.multiple_occurs() && values_list.len() > 1 {
-                        return Some(MatchError::NotMultipleArgument(
-                            level,
-                            param.render_long_name(),
-                        ));
+                        return Some(MatchError::NotMultipleArgument(level, param.long_name()));
                     }
                 }
             }
+
+            for name in check_flag_option_bind_envs {
+                if let Some(param) = cmd.flag_option_params.iter().find(|v| v.id() == name) {
+                    let values = &flag_option_bind_envs[name];
+                    let mut is_valid = true;
+                    let mut choice_values = vec![];
+                    let (min, _) = param.num_args();
+                    if param.is_flag() {
+                        if !is_bool_value(values[0]) {
+                            is_valid = false;
+                        }
+                    } else if min > 1 {
+                        is_valid = false;
+                    }
+                    if !is_valid {
+                        return Some(MatchError::InvalidBindEnvironment(
+                            level,
+                            values[0].to_string(),
+                            param.bind_env().unwrap_or_default(),
+                            param.long_name(),
+                            choice_values,
+                        ));
+                    }
+                    if let Some(choices) = get_param_choice(param.choice(), &choices_fn_values) {
+                        choice_values = choices.to_vec();
+                        for value in values.iter() {
+                            if !choices.contains(&value.to_string()) {
+                                return Some(MatchError::InvalidBindEnvironment(
+                                    level,
+                                    value.to_string(),
+                                    param.bind_env().unwrap_or_default(),
+                                    param.long_name(),
+                                    choice_values,
+                                ));
+                            }
+                        }
+                    }
+                }
+            }
+
             if !missing_flag_options.is_empty() {
                 let missing_flag_options: Vec<String> = missing_flag_options
                     .iter()
@@ -653,7 +802,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
         for (i, param) in last_cmd.positional_params.iter().enumerate() {
             if let (Some(values), Some(choices)) = (
                 positional_values.get(i),
-                get_param_choice(&param.data.choice, &choices_fn_values),
+                get_param_choice(param.choice(), &choices_fn_values),
             ) {
                 for value in values.iter() {
                     if !choices.contains(&value.to_string()) {
@@ -668,11 +817,26 @@ impl<'a, 'b> Matcher<'a, 'b> {
             }
         }
         if positional_params_len > positional_values_len {
-            let missing_positionals: Vec<_> = last_cmd.positional_params[positional_values_len..]
-                .iter()
-                .filter(|param| param.required())
-                .map(|v| v.render_notation())
-                .collect();
+            let mut missing_positionals = vec![];
+            for param in &last_cmd.positional_params[positional_values_len..] {
+                if let Some(values) = bind_envs.positionals.get(param.id()) {
+                    if let Some(choices) = get_param_choice(param.choice(), &choices_fn_values) {
+                        for value in values.iter() {
+                            if !choices.contains(&value.to_string()) {
+                                return Some(MatchError::InvalidBindEnvironment(
+                                    level,
+                                    value.to_string(),
+                                    param.bind_env().unwrap_or_default(),
+                                    param.render_notation(),
+                                    choices.to_vec(),
+                                ));
+                            }
+                        }
+                    }
+                } else if param.required() {
+                    missing_positionals.push(param.render_notation())
+                }
+            }
             if !missing_positionals.is_empty() {
                 return Some(MatchError::MissingRequiredArguments(
                     level,
@@ -693,7 +857,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
 
         for param in &last_cmd.env_params {
             if let (Some(choices), Some(value)) = (
-                get_param_choice(&param.data.choice, &choices_fn_values),
+                get_param_choice(param.choice(), &choices_fn_values),
                 self.envs.get(param.id()),
             ) {
                 if !choices.contains(&value.to_string()) {
@@ -710,12 +874,19 @@ impl<'a, 'b> Matcher<'a, 'b> {
         None
     }
 
-    fn run_choices_fns(&'a self) -> Option<HashMap<&'a str, Vec<String>>> {
+    fn run_choices_fns<'x>(
+        &'x self,
+        bind_envs: &BindEnvs<'a, 'x>,
+    ) -> Option<HashMap<&'a str, Vec<String>>> {
+        let fns: Vec<_> = {
+            let mut fns = self.choice_fns.clone();
+            fns.extend(bind_envs.choice_fns.iter());
+            fns.into_iter().collect()
+        };
         let script_path = self.script_path.as_ref()?;
         let mut choices_fn_values = HashMap::new();
-        let fns: Vec<&str> = self.choice_fns.iter().copied().collect();
         let mut envs = HashMap::new();
-        envs.insert("ARGC_OS".into(), env::consts::OS.to_string());
+        envs.insert("ARGC_OS".into(), get_os());
         let outputs = run_param_fns(script_path, &fns, self.args, envs)?;
         for (i, output) in outputs.into_iter().enumerate() {
             let choices = output
@@ -838,6 +1009,20 @@ impl<'a, 'b> Matcher<'a, 'b> {
   [possible values: {list}]"###
                 )
             }
+            MatchError::InvalidBindEnvironment(_level, value, env_name, name, choices) => {
+                exit = 1;
+                if choices.is_empty() {
+                    format!(
+                        r###"error: environment variable `{env_name}` has invalid value for param '{name}'"###
+                    )
+                } else {
+                    let list = choices.join(", ");
+                    format!(
+                        r###"error: invalid value `{value}` for environment variable `{env_name}` that bound to `{name}`
+  [possible values: {list}]"###
+                    )
+                }
+            }
             MatchError::InvalidEnvironment(_level, value, name, choices) => {
                 exit = 1;
                 let list = choices.join(", ");
@@ -885,7 +1070,13 @@ impl<'a, 'b> Matcher<'a, 'b> {
                     CompColor::of_option()
                 };
                 for v in param.list_names() {
-                    output.push((v, describe.to_string(), param.prefixed(), kind))
+                    let nospace = param.prefixed() || param.assigned();
+                    let v = if param.assigned() && v.len() > 2 {
+                        format!("{v}=")
+                    } else {
+                        v
+                    };
+                    output.push((v, describe.to_string(), nospace, kind))
                 }
             }
         }
@@ -900,6 +1091,25 @@ impl<'a, 'b> Matcher<'a, 'b> {
 /// (value, description, nospace, color)
 pub(crate) type CompItem = (String, String, bool, CompColor);
 
+#[derive(Debug)]
+struct BindEnvs<'a, 'x> {
+    flag_options: Vec<BindEnvMap<'a, 'x>>,
+    positionals: BindEnvMap<'a, 'x>,
+    choice_fns: HashSet<&'a str>,
+}
+
+impl<'a, 'x> BindEnvs<'a, 'x> {
+    fn new(len: usize) -> Self {
+        Self {
+            flag_options: vec![HashMap::new(); len],
+            positionals: HashMap::new(),
+            choice_fns: HashSet::new(),
+        }
+    }
+}
+
+type BindEnvMap<'a, 'x> = HashMap<&'a str, Vec<&'x str>>;
+
 fn find_subcommand<'a>(
     cmd: &'a Command,
     arg: &str,
@@ -938,9 +1148,15 @@ fn add_positional_arg<'a>(
     }
 }
 
-fn take_value_args<'a>(args: &'a [String], start: usize, len: usize, signs: &str) -> Vec<&'a str> {
+fn take_value_args<'a>(
+    args: &'a [String],
+    start: usize,
+    len: usize,
+    signs: &str,
+    assigned: bool,
+) -> Vec<&'a str> {
     let mut output = vec![];
-    if len == 0 {
+    if assigned || len == 0 {
         return output;
     }
     let end = (start + len).min(args.len());
@@ -1008,11 +1224,11 @@ fn match_flag_option<'a, 'b>(
     } else if let Some(prefix) = param.match_prefix(arg) {
         let args_len = args.len();
         let prefix_len = prefix.len();
-        let value_args = take_value_args(args, *arg_index + 1, 1, signs);
+        let value_args = take_value_args(args, *arg_index + 1, 1, signs, param.assigned());
         let take_empty = value_args.is_empty();
         *arg_index += value_args.len();
         let values = if take_empty { vec!["1"] } else { value_args };
-        if *arg_index == args_len - 1 {
+        if !param.assigned() && *arg_index == args_len - 1 {
             *arg_comp = ArgComp::OptionValue(param.id().to_string(), 0);
             if take_empty {
                 *split_last_arg_at = Some(prefix_len);
@@ -1020,13 +1236,13 @@ fn match_flag_option<'a, 'b>(
         }
         flag_option_args.push((arg, values, Some(param.id())));
     } else {
-        let values_max = param.args_range().1;
+        let values_max = param.num_args().1;
         let args_len = args.len();
-        let value_args = take_value_args(args, *arg_index + 1, values_max, signs);
+        let value_args = take_value_args(args, *arg_index + 1, values_max, signs, param.assigned());
         *arg_index += value_args.len();
         if *arg_index == args_len - 1 {
             if *arg_comp != ArgComp::FlagOrOption {
-                if param.is_option() && value_args.len() <= values_max {
+                if param.is_option() && !param.assigned() && value_args.len() <= values_max {
                     *arg_comp = ArgComp::OptionValue(
                         param.id().to_string(),
                         value_args.len().saturating_sub(1),
@@ -1158,15 +1374,15 @@ fn comp_symbol(cmd: &Command, ch: char) -> Vec<CompItem> {
 
 fn comp_flag_option(param: &FlagOptionParam, index: usize) -> Vec<CompItem> {
     let value_name = param
-        .notations
+        .notations()
         .get(index)
         .map(|v| v.as_str())
-        .unwrap_or_else(|| param.notations.last().unwrap());
-    comp_param(param.describe_oneline(), value_name, &param.data)
+        .unwrap_or_else(|| param.notations().last().unwrap());
+    comp_param(param.describe_oneline(), value_name, param.data())
 }
 
 fn comp_positional(param: &PositionalParam) -> Vec<CompItem> {
-    comp_param(param.describe_oneline(), &param.notation, &param.data)
+    comp_param(param.describe_oneline(), param.notation(), param.data())
 }
 
 fn comp_param(describe: &str, value_name: &str, data: &ParamData) -> Vec<CompItem> {
@@ -1209,7 +1425,7 @@ fn comp_param(describe: &str, value_name: &str, data: &ParamData) -> Vec<CompIte
 }
 
 fn get_param_choice<'a, 'b: 'a>(
-    choice: &'a Option<ChoiceValue>,
+    choice: Option<&'a ChoiceValue>,
     choices_fn_values: &'a HashMap<&str, Vec<String>>,
 ) -> Option<&'a Vec<String>> {
     match choice {
@@ -1225,10 +1441,14 @@ fn get_param_choice<'a, 'b: 'a>(
     }
 }
 
-fn delimit_arg_values<'b, T: Param>(param: &T, values: &[&'b str]) -> Vec<&'b str> {
-    if let Some(delimiter) = param.args_delimiter() {
+fn delimit_arg_values<'x, T: Param>(param: &T, values: &[&'x str]) -> Vec<&'x str> {
+    if let Some(delimiter) = param.delimiter() {
         values.iter().flat_map(|v| v.split(delimiter)).collect()
     } else {
         values.to_vec()
     }
 }
+
+fn is_bool_value(value: &str) -> bool {
+    matches!(value, "true" | "false" | "0" | "1")
+}
