@@ -1,8 +1,5 @@
-use crate::matcher::FlagOptionArg;
-use crate::utils::{
-    argc_var_name, escape_shell_words, is_choice_value_terminate, is_default_value_terminate,
-    to_cobol_case, MAX_ARGS,
-};
+use crate::parser::{is_choice_value_terminate, is_default_value_terminate};
+use crate::utils::{argc_var_name, escape_shell_words, sanitize_var_name, to_cobol_case, MAX_ARGS};
 use crate::ArgcValue;
 
 use anyhow::{bail, Result};
@@ -11,13 +8,14 @@ use serde::Serialize;
 
 pub(crate) trait Param {
     fn data(&self) -> &ParamData;
+    fn data_mut(&mut self) -> &mut ParamData;
     fn id(&self) -> &str;
     fn var_name(&self) -> String;
-    fn describe(&self) -> &str;
     fn tag_name(&self) -> &str;
     fn guard(&self) -> Result<()>;
     fn multiple_values(&self) -> bool;
     fn render_source(&self) -> String;
+
     fn describe_oneline(&self) -> &str {
         match self.describe().split_once('\n') {
             Some((v, _)) => v,
@@ -25,24 +23,23 @@ pub(crate) trait Param {
         }
     }
     fn render_describe(&self) -> String {
-        self.data().render_describe(self.describe())
+        self.data().render_describe(self.describe(), self.id())
+    }
+    fn describe(&self) -> &str {
+        &self.data().describe
+    }
+    fn describe_mut(&mut self) -> &mut String {
+        &mut self.data_mut().describe
     }
-
     fn required(&self) -> bool {
         self.data().required()
     }
-    fn multiple_occurs(&self) -> bool {
-        self.data().multiple_occurs()
-    }
-    fn args_delimiter(&self) -> Option<char> {
+    fn delimiter(&self) -> Option<char> {
         self.data().args_delimiter()
     }
     fn terminated(&self) -> bool {
         self.data().terminated()
     }
-    fn prefixed(&self) -> bool {
-        self.data().prefixed()
-    }
     fn choice(&self) -> Option<&ChoiceValue> {
         self.data().choice.as_ref()
     }
@@ -55,25 +52,28 @@ pub(crate) trait Param {
     fn default(&self) -> Option<&DefaultValue> {
         self.data().default.as_ref()
     }
+    fn default_fn(&self) -> Option<&String> {
+        self.data().default_fn()
+    }
     fn default_value(&self) -> Option<&String> {
         self.data().default_value()
     }
-    fn default_fn(&self) -> Option<&String> {
-        self.data().default_fn()
+    fn bind_env(&self) -> Option<String> {
+        self.data().normalize_bind_env(self.id())
     }
 }
 
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub(crate) struct FlagOptionParam {
-    pub(crate) data: ParamData,
-    pub(crate) describe: String,
-    pub(crate) is_flag: bool,
-    pub(crate) short: Option<String>,
-    pub(crate) long_prefix: String,
-    pub(crate) id: String,
-    pub(crate) raw_notations: Vec<String>,
-    pub(crate) notations: Vec<String>,
-    pub(crate) inherit: bool,
+    data: ParamData,
+    id: String,
+    short: Option<String>,
+    long_prefix: String,
+    prefixed: bool,
+    assigned: bool,
+    raw_notations: Vec<String>,
+    notations: Vec<String>,
+    inherited: bool,
 }
 
 impl Param for FlagOptionParam {
@@ -81,6 +81,10 @@ impl Param for FlagOptionParam {
         &self.data
     }
 
+    fn data_mut(&mut self) -> &mut ParamData {
+        &mut self.data
+    }
+
     fn id(&self) -> &str {
         &self.id
     }
@@ -89,10 +93,6 @@ impl Param for FlagOptionParam {
         argc_var_name(self.id())
     }
 
-    fn describe(&self) -> &str {
-        &self.describe
-    }
-
     fn tag_name(&self) -> &str {
         if self.is_flag() {
             "@flag"
@@ -103,18 +103,24 @@ impl Param for FlagOptionParam {
 
     fn guard(&self) -> Result<()> {
         if self.notations.len() > 1 {
-            if self.prefixed() {
+            if self.assigned {
+                bail!("cannot combine assign and multiple notations")
+            }
+            if self.prefixed {
                 bail!("cannot combine prefix and multiple notations")
             }
-            if self.args_delimiter().is_some() {
+            if self.delimiter().is_some() {
                 bail!("cannot combine delmiter and multiple notations")
             }
         }
+        if self.prefixed && self.bind_env().is_some() {
+            bail!("cannot bind env with prefixed options")
+        }
         Ok(())
     }
 
     fn multiple_values(&self) -> bool {
-        self.multiple_occurs() || self.args_range().1 > 1
+        self.data().multiple() || self.num_args().1 > 1
     }
 
     fn render_source(&self) -> String {
@@ -122,30 +128,59 @@ impl Param for FlagOptionParam {
         if let Some(short) = &self.short {
             output.push(short.to_string());
         };
+        let mut name_suffix = String::new();
+        if self.prefixed || self.data.name.ends_with('-') {
+            name_suffix.push('-');
+        }
+        if self.assigned || self.data.name.ends_with(':') {
+            name_suffix.push(':');
+        }
         output.push(format!(
             "{}{}",
             self.long_prefix,
-            self.data.render_name_value()
+            self.data.render_source_of_name_value(&name_suffix)
         ));
+
+        if let Some(env) = &self.data.env {
+            match env {
+                Some(v) => output.push(format!("${v}")),
+                None => output.push("$$".into()),
+            }
+        }
+
         for raw_notation in &self.raw_notations {
             output.push(format!("<{}>", raw_notation));
         }
-        if !self.describe.is_empty() {
-            output.push(self.describe.clone());
+
+        if !self.data.describe.is_empty() {
+            output.push(self.data.describe.clone());
         }
+
         output.join(" ")
     }
 }
 
 impl FlagOptionParam {
     pub(crate) fn new(
-        data: ParamData,
-        describe: &str,
-        is_flag: bool,
+        mut data: ParamData,
+        flag: bool,
         short: Option<&str>,
         long_prefix: &str,
         row_notations: &[&str],
     ) -> Self {
+        let (mut prefixed, mut assigned) = (false, false);
+        if let Some(new_name) = data.name.strip_suffix("::") {
+            data.name = format!("{new_name}:");
+        } else if let Some(new_name) = data.name.strip_suffix(':') {
+            data.name = new_name.to_string();
+            assigned = true;
+        }
+        if let Some(new_name) = data.name.strip_suffix("--") {
+            data.name = format!("{new_name}-");
+        } else if let Some(new_name) = data.name.strip_suffix('-') {
+            data.name = new_name.to_string();
+            prefixed = true;
+        }
         let name = data.name.clone();
         let id = if long_prefix.starts_with('+') {
             format!("plus_{}", name)
@@ -153,7 +188,7 @@ impl FlagOptionParam {
             name.clone()
         };
         let raw_notations: Vec<String> = row_notations.iter().map(|v| v.to_string()).collect();
-        let mut notations = if is_flag {
+        let mut notations = if flag {
             vec![]
         } else if raw_notations.is_empty() {
             vec![to_cobol_case(&name)]
@@ -165,48 +200,74 @@ impl FlagOptionParam {
             last_arg.push('~')
         }
         Self {
-            describe: describe.to_string(),
-            is_flag,
             short: short.map(|v| v.to_string()),
             long_prefix: long_prefix.to_string(),
+            prefixed,
+            assigned,
             data,
             id,
             raw_notations,
             notations,
-            inherit: false,
+            inherited: false,
         }
     }
 
     pub(crate) fn export(&self) -> FlagOptionValue {
         FlagOptionValue {
             id: self.id().to_string(),
-            long_name: self.render_long_name(),
+            long_name: self.long_name(),
             short_name: self.short.clone(),
-            describe: self.describe.clone(),
-            is_flag: self.is_flag,
+            describe: self.describe().to_string(),
+            flag: self.is_flag(),
             notations: self.notations.clone(),
             required: self.required(),
             multiple_values: self.multiple_values(),
             multiple_occurs: self.multiple_occurs(),
-            args_range: self.args_range(),
-            args_delimiter: self.args_delimiter(),
+            num_args: self.num_args(),
+            delimiter: self.delimiter(),
             terminated: self.terminated(),
-            prefixed: self.prefixed(),
+            prefixed: self.prefixed,
+            assigned: self.assigned,
             default: self.data().default.clone(),
             choice: self.data().choice.clone(),
-            inherit: self.inherit,
+            env: self.bind_env(),
+            inherited: self.inherited,
         }
     }
 
     pub(crate) fn is_flag(&self) -> bool {
-        self.is_flag
+        self.notations.is_empty()
     }
 
     pub(crate) fn is_option(&self) -> bool {
         !self.is_flag()
     }
 
-    pub(crate) fn args_range(&self) -> (usize, usize) {
+    pub(crate) fn assigned(&self) -> bool {
+        self.assigned
+    }
+
+    pub(crate) fn prefixed(&self) -> bool {
+        self.prefixed
+    }
+
+    pub(crate) fn short(&self) -> &Option<String> {
+        &self.short
+    }
+
+    pub(crate) fn long_prefix(&self) -> &str {
+        &self.long_prefix
+    }
+
+    pub(crate) fn notations(&self) -> &[String] {
+        &self.notations
+    }
+
+    pub(crate) fn multiple_occurs(&self) -> bool {
+        self.data().multiple()
+    }
+
+    pub(crate) fn num_args(&self) -> (usize, usize) {
         let len = self.notations.len();
         if self.terminated()
             || self
@@ -233,7 +294,7 @@ impl FlagOptionParam {
             .and_then(|name| ['*', '+', '?'].into_iter().find(|v| name.ends_with(*v)))
     }
 
-    pub(crate) fn render_long_name(&self) -> String {
+    pub(crate) fn long_name(&self) -> String {
         format!("{}{}", self.long_prefix, self.data.name)
     }
 
@@ -242,34 +303,10 @@ impl FlagOptionParam {
     }
 
     pub(crate) fn render_name_notations(&self) -> String {
-        let mut output = self.render_long_name();
+        let mut output = self.long_name();
         if !self.is_flag() {
-            output.push(' ');
-            output.push_str(&self.render_notations());
-        }
-        output
-    }
-
-    pub(crate) fn render_body(&self) -> String {
-        let mut output = String::new();
-        if self.short.is_none() && self.long_prefix.len() == 1 && self.data.name.len() == 1 {
-            output.push_str(&self.render_long_name());
-        } else {
-            if let Some(short) = &self.short {
-                output.push_str(&format!("{short}, "))
-            } else {
-                output.push_str("    ")
-            };
-            output.push_str(&format!("{:>2}", self.long_prefix));
-            output.push_str(&self.data.name);
-        }
-
-        if self.is_flag() {
-            if self.multiple_occurs() {
-                output.push_str("...")
-            }
-        } else {
-            output.push(' ');
+            let ch = if self.assigned { '=' } else { ' ' };
+            output.push(ch);
             output.push_str(&self.render_notations());
         }
         output
@@ -296,29 +333,53 @@ impl FlagOptionParam {
         list.join(" ")
     }
 
-    pub(crate) fn to_argc_value(&self, args: &[FlagOptionArg]) -> Option<ArgcValue> {
+    pub(crate) fn render_body(&self) -> String {
+        let mut output = String::new();
+        if self.short.is_none() && self.long_prefix.len() == 1 && self.data.name.len() == 1 {
+            output.push_str(&self.long_name());
+        } else {
+            if let Some(short) = &self.short {
+                output.push_str(&format!("{short}, "))
+            } else {
+                output.push_str("    ")
+            };
+            output.push_str(&format!("{:>2}", self.long_prefix));
+            output.push_str(&self.data.name);
+        }
+
+        if self.is_flag() {
+            if self.multiple_occurs() {
+                output.push_str("...")
+            }
+        } else {
+            let ch = if self.assigned { '=' } else { ' ' };
+            output.push(ch);
+            output.push_str(&self.render_notations());
+        }
+        output
+    }
+
+    pub(crate) fn to_argc_value(&self, args: &[(&str, &[&str])]) -> Option<ArgcValue> {
         let id = self.id().to_string();
-        if self.prefixed() {
+        if self.prefixed {
             let mut map: IndexMap<String, Vec<String>> = IndexMap::new();
-            for (arg, value, name) in args.iter() {
-                if name == &Some(id.as_str()) {
-                    if let Some(arg_suffix) = self
-                        .list_names()
-                        .into_iter()
-                        .find_map(|v| arg.strip_prefix(&v))
-                    {
-                        let key = match arg_suffix.split_once('=') {
-                            Some((arg_suffix, _)) => arg_suffix,
-                            None => arg_suffix,
-                        };
-                        if let Some(values) = map.get_mut(key) {
-                            values.extend(value.iter().map(|v| v.to_string()));
-                        } else {
-                            map.insert(
-                                key.to_string(),
-                                value.iter().map(|v| v.to_string()).collect(),
-                            );
-                        }
+            for (arg, value) in args {
+                if let Some(arg_suffix) = self
+                    .list_names()
+                    .into_iter()
+                    .find_map(|v| arg.strip_prefix(&v))
+                {
+                    let key = match arg_suffix.split_once('=') {
+                        Some((arg_suffix, _)) => arg_suffix,
+                        None => arg_suffix,
+                    };
+                    if let Some(values) = map.get_mut(key) {
+                        values.extend(value.iter().map(|v| v.to_string()));
+                    } else {
+                        map.insert(
+                            key.to_string(),
+                            value.iter().map(|v| v.to_string()).collect(),
+                        );
                     }
                 }
             }
@@ -328,17 +389,8 @@ impl FlagOptionParam {
                 Some(ArgcValue::Map(id, map))
             }
         } else {
-            let values: Vec<&[&str]> = args
-                .iter()
-                .filter_map(|(_, value, name)| {
-                    if name == &Some(self.id()) {
-                        Some(value.as_slice())
-                    } else {
-                        None
-                    }
-                })
-                .collect();
-            if self.is_flag {
+            let values: Vec<&[&str]> = args.iter().map(|(_, value)| *value).collect();
+            if self.is_flag() {
                 if values.is_empty() {
                     None
                 } else {
@@ -374,8 +426,28 @@ impl FlagOptionParam {
         }
     }
 
+    pub(crate) fn set_inherit(&mut self) {
+        self.inherited = true;
+    }
+
+    pub(crate) fn create_help_flag(short: Option<&str>, long_prefix: &str, describe: &str) -> Self {
+        let mut param_data = ParamData::new("help");
+        param_data.describe = describe.to_string();
+        FlagOptionParam::new(param_data, true, short, long_prefix, &[])
+    }
+
+    pub(crate) fn create_version_flag(
+        short: Option<&str>,
+        long_prefix: &str,
+        describe: &str,
+    ) -> Self {
+        let mut param_data = ParamData::new("version");
+        param_data.describe = describe.to_string();
+        FlagOptionParam::new(param_data, true, short, long_prefix, &[])
+    }
+
     pub(crate) fn match_prefix<'a>(&self, arg: &'a str) -> Option<&'a str> {
-        if self.prefixed() {
+        if self.prefixed {
             self.list_names().iter().find_map(|v| {
                 if arg.starts_with(v) {
                     Some(&arg[..v.len()])
@@ -391,7 +463,7 @@ impl FlagOptionParam {
     pub(crate) fn is_match(&self, name: &str) -> bool {
         self.id() == name
             || self.list_names().iter().any(|v| {
-                if self.prefixed() {
+                if self.prefixed {
                     name.starts_with(v)
                 } else {
                     v == name
@@ -401,7 +473,7 @@ impl FlagOptionParam {
 
     pub(crate) fn list_names(&self) -> Vec<String> {
         let mut output = vec![];
-        output.push(self.render_long_name());
+        output.push(self.long_name());
         if let Some(short) = &self.short {
             output.push(short.clone());
         }
@@ -415,26 +487,27 @@ pub struct FlagOptionValue {
     pub long_name: String,
     pub short_name: Option<String>,
     pub describe: String,
-    pub is_flag: bool,
+    pub flag: bool,
     pub notations: Vec<String>,
     pub required: bool,
     pub multiple_values: bool,
     pub multiple_occurs: bool,
-    pub args_range: (usize, usize),
-    pub args_delimiter: Option<char>,
+    pub num_args: (usize, usize),
+    pub delimiter: Option<char>,
     pub terminated: bool,
     pub prefixed: bool,
+    pub assigned: bool,
     pub default: Option<DefaultValue>,
     pub choice: Option<ChoiceValue>,
-    pub inherit: bool,
+    pub env: Option<String>,
+    pub inherited: bool,
 }
 
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub(crate) struct PositionalParam {
-    pub(crate) data: ParamData,
-    pub(crate) describe: String,
-    pub(crate) raw_notation: Option<String>,
-    pub(crate) notation: String,
+    data: ParamData,
+    raw_notation: Option<String>,
+    notation: String,
 }
 
 impl Param for PositionalParam {
@@ -442,6 +515,10 @@ impl Param for PositionalParam {
         &self.data
     }
 
+    fn data_mut(&mut self) -> &mut ParamData {
+        &mut self.data
+    }
+
     fn id(&self) -> &str {
         &self.data().name
     }
@@ -450,10 +527,6 @@ impl Param for PositionalParam {
         argc_var_name(self.id())
     }
 
-    fn describe(&self) -> &str {
-        &self.describe
-    }
-
     fn tag_name(&self) -> &str {
         "@arg"
     }
@@ -463,28 +536,33 @@ impl Param for PositionalParam {
     }
 
     fn multiple_values(&self) -> bool {
-        self.multiple_occurs() || self.terminated()
+        self.data.multiple() || self.terminated()
     }
 
     fn render_source(&self) -> String {
-        let mut output = vec![];
-        output.push(self.data.render_name_value());
+        let mut output = vec![self.data.render_source_of_name_value("")];
+
+        if let Some(env) = &self.data.env {
+            match env {
+                Some(v) => output.push(format!("${v}")),
+                None => output.push("$$".into()),
+            }
+        }
         if let Some(raw_notation) = self.raw_notation.as_ref() {
             output.push(format!("<{}>", raw_notation));
         }
-        if !self.describe.is_empty() {
-            output.push(self.describe.clone());
+        if !self.data.describe.is_empty() {
+            output.push(self.data.describe.clone());
         }
         output.join(" ")
     }
 }
 
 impl PositionalParam {
-    pub(crate) fn new(data: ParamData, describe: &str, raw_notation: Option<&str>) -> Self {
+    pub(crate) fn new(data: ParamData, raw_notation: Option<&str>) -> Self {
         let name = data.name.clone();
         PositionalParam {
             data,
-            describe: describe.to_string(),
             raw_notation: raw_notation.map(|v| v.to_string()),
             notation: raw_notation
                 .or(Some(&name))
@@ -496,19 +574,22 @@ impl PositionalParam {
     pub(crate) fn export(&self) -> PositionalValue {
         PositionalValue {
             id: self.id().to_string(),
-            describe: self.describe.clone(),
+            describe: self.describe().to_string(),
             notation: self.notation.clone(),
             required: self.required(),
-            multiple_values: self.multiple_values(),
-            multiple_occurs: self.multiple_occurs(),
-            args_delimiter: self.args_delimiter(),
+            multiple: self.multiple_values(),
+            delimiter: self.delimiter(),
             terminated: self.terminated(),
-            prefixed: self.prefixed(),
             default: self.data().default.clone(),
             choice: self.data().choice.clone(),
+            env: self.bind_env(),
         }
     }
 
+    pub(crate) fn notation(&self) -> &str {
+        &self.notation
+    }
+
     pub(crate) fn render_notation(&self) -> String {
         let name: &String = &self.notation;
         match (self.required(), self.multiple_values()) {
@@ -547,20 +628,18 @@ pub struct PositionalValue {
     pub describe: String,
     pub notation: String,
     pub required: bool,
-    pub multiple_values: bool,
-    pub multiple_occurs: bool,
-    pub args_delimiter: Option<char>,
+    pub multiple: bool,
+    pub delimiter: Option<char>,
     pub terminated: bool,
-    pub prefixed: bool,
     pub default: Option<DefaultValue>,
     pub choice: Option<ChoiceValue>,
+    pub env: Option<String>,
 }
 
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub(crate) struct EnvParam {
-    pub(crate) data: ParamData,
-    pub(crate) describe: String,
-    pub(crate) inherit: bool,
+    data: ParamData,
+    inherited: bool,
 }
 
 impl Param for EnvParam {
@@ -568,6 +647,10 @@ impl Param for EnvParam {
         &self.data
     }
 
+    fn data_mut(&mut self) -> &mut ParamData {
+        &mut self.data
+    }
+
     fn id(&self) -> &str {
         &self.data().name
     }
@@ -576,10 +659,6 @@ impl Param for EnvParam {
         self.id().to_string()
     }
 
-    fn describe(&self) -> &str {
-        &self.describe
-    }
-
     fn tag_name(&self) -> &str {
         "@env"
     }
@@ -588,6 +667,9 @@ impl Param for EnvParam {
         if !matches!(self.data().modifer, Modifier::Optional | Modifier::Required) {
             bail!("can only be a single value")
         }
+        if self.data.env.is_some() {
+            bail!("cannot bind another env")
+        }
         Ok(())
     }
 
@@ -596,32 +678,36 @@ impl Param for EnvParam {
     }
 
     fn render_source(&self) -> String {
-        let mut output = vec![];
-        output.push(self.data.render_name_value());
-        if !self.describe.is_empty() {
-            output.push(self.describe.clone());
+        let mut output = vec![self.data.render_source_of_name_value("")];
+        if let Some(env) = &self.data.env {
+            match env {
+                Some(v) => output.push(format!("${v}")),
+                None => output.push("$$".into()),
+            }
+        }
+        if !self.data.describe.is_empty() {
+            output.push(self.data.describe.clone());
         }
         output.join(" ")
     }
 }
 
 impl EnvParam {
-    pub(crate) fn new(data: ParamData, describe: &str) -> Self {
+    pub(crate) fn new(data: ParamData) -> Self {
         Self {
-            describe: describe.to_string(),
             data,
-            inherit: false,
+            inherited: false,
         }
     }
 
     pub(crate) fn export(&self) -> EnvValue {
         EnvValue {
             id: self.id().to_string(),
-            describe: self.describe.clone(),
+            describe: self.describe().to_string(),
             required: self.required(),
             default: self.data().default.clone(),
             choice: self.data().choice.clone(),
-            inherit: self.inherit,
+            inherited: self.inherited,
         }
     }
 
@@ -639,6 +725,10 @@ impl EnvParam {
         };
         Some(value)
     }
+
+    pub(crate) fn set_inherit(&mut self) {
+        self.inherited = true;
+    }
 }
 
 #[derive(Debug, Serialize)]
@@ -648,24 +738,28 @@ pub struct EnvValue {
     pub required: bool,
     pub default: Option<DefaultValue>,
     pub choice: Option<ChoiceValue>,
-    pub inherit: bool,
+    pub inherited: bool,
 }
 
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub(crate) struct ParamData {
     pub(crate) name: String,
+    pub(crate) describe: String,
     pub(crate) choice: Option<ChoiceValue>,
     pub(crate) default: Option<DefaultValue>,
     pub(crate) modifer: Modifier,
+    pub(crate) env: Option<Option<String>>,
 }
 
 impl ParamData {
     pub(crate) fn new(name: &str) -> Self {
         Self {
             name: name.to_string(),
+            describe: String::new(),
             choice: None,
             default: None,
             modifer: Modifier::Optional,
+            env: None,
         }
     }
 
@@ -673,8 +767,8 @@ impl ParamData {
         self.modifer.required() && self.default.is_none()
     }
 
-    pub(crate) fn multiple_occurs(&self) -> bool {
-        self.modifer.multiple_occurs()
+    pub(crate) fn multiple(&self) -> bool {
+        self.modifer.multiple()
     }
 
     pub(crate) fn args_delimiter(&self) -> Option<char> {
@@ -688,10 +782,6 @@ impl ParamData {
         matches!(self.modifer, Modifier::Terminated)
     }
 
-    pub(crate) fn prefixed(&self) -> bool {
-        matches!(self.modifer, Modifier::Prefixed | Modifier::MultiPrefixed)
-    }
-
     pub(crate) fn choice_fn(&self) -> Option<(&String, &bool)> {
         match &self.choice {
             Some(ChoiceValue::Fn(f, v)) => Some((f, v)),
@@ -713,7 +803,6 @@ impl ParamData {
         }
     }
 
-    #[allow(unused)]
     pub(crate) fn default_value(&self) -> Option<&String> {
         match &self.default {
             Some(DefaultValue::Value(v)) => Some(v),
@@ -721,8 +810,16 @@ impl ParamData {
         }
     }
 
-    pub(crate) fn render_name_value(&self) -> String {
-        let mut output = self.name.to_string();
+    pub(crate) fn normalize_bind_env(&self, id: &str) -> Option<String> {
+        let env = match self.env.as_ref()? {
+            Some(v) => v.clone(),
+            None => sanitize_var_name(id).to_uppercase(),
+        };
+        Some(env)
+    }
+
+    pub(crate) fn render_source_of_name_value(&self, name_suffix: &str) -> String {
+        let mut output = format!("{}{name_suffix}", self.name);
         output.push_str(&self.modifer.render());
         match (&self.choice, &self.default) {
             (Some(ChoiceValue::Values(values)), None) => {
@@ -746,7 +843,7 @@ impl ParamData {
         output
     }
 
-    pub(crate) fn render_describe(&self, describe: &str) -> String {
+    pub(crate) fn render_describe(&self, describe: &str, id: &str) -> String {
         let mut output = describe.to_string();
         let multiline = output.contains('\n');
         let sep = if multiline { '\n' } else { ' ' };
@@ -763,6 +860,12 @@ impl ParamData {
             let values: Vec<String> = values.iter().map(|v| escape_shell_words(v)).collect();
             output.push_str(&format!("[possible values: {}]", values.join(", ")));
         }
+        if let Some(env) = self.normalize_bind_env(id) {
+            if !output.is_empty() {
+                output.push(sep)
+            }
+            output.push_str(&format!("[env: {env}]"));
+        }
         output
     }
 
@@ -799,12 +902,10 @@ pub(crate) enum Modifier {
     DelimiterOptional(char),
     DelimiterRequired(char),
     Terminated,
-    Prefixed,
-    MultiPrefixed,
 }
 
 impl Modifier {
-    pub(crate) fn multiple_occurs(&self) -> bool {
+    pub(crate) fn multiple(&self) -> bool {
         match self {
             Self::Optional => false,
             Self::Required => false,
@@ -813,8 +914,6 @@ impl Modifier {
             Self::DelimiterOptional(_) => true,
             Self::DelimiterRequired(_) => true,
             Self::Terminated => false,
-            Self::Prefixed => false,
-            Self::MultiPrefixed => true,
         }
     }
 
@@ -827,8 +926,6 @@ impl Modifier {
             Self::DelimiterOptional(_) => false,
             Self::DelimiterRequired(_) => true,
             Self::Terminated => false,
-            Self::Prefixed => false,
-            Self::MultiPrefixed => false,
         }
     }
 
@@ -841,8 +938,6 @@ impl Modifier {
             Self::DelimiterOptional(c) => format!("*{c}"),
             Self::DelimiterRequired(c) => format!("+{c}"),
             Self::Terminated => "~".to_string(),
-            Self::Prefixed => "-".to_string(),
-            Self::MultiPrefixed => "-*".to_string(),
         }
     }
 }
