@@ -1,7 +1,8 @@
 use crate::param::{
-    ChoiceValue, DefaultValue, EnvParam, FlagOptionParam, Modifier, ParamData, PositionalParam,
+    ChoiceValue, DefaultValue, EnvParam, FlagOptionParam, Modifier, Param, ParamData,
+    PositionalParam,
 };
-use crate::utils::{is_choice_value_terminate, is_default_value_terminate};
+use crate::utils::is_special_var_char;
 use crate::Result;
 use anyhow::bail;
 use nom::{
@@ -86,17 +87,17 @@ pub(crate) fn parse(source: &str) -> Result<Vec<Event>> {
                             }
                             EventData::Env(mut param) => {
                                 line_idx +=
-                                    take_comment_lines(&lines, line_idx + 1, &mut param.describe);
+                                    take_comment_lines(&lines, line_idx + 1, param.describe_mut());
                                 EventData::Env(param)
                             }
                             EventData::FlagOption(mut param) => {
                                 line_idx +=
-                                    take_comment_lines(&lines, line_idx + 1, &mut param.describe);
+                                    take_comment_lines(&lines, line_idx + 1, param.describe_mut());
                                 EventData::FlagOption(param)
                             }
                             EventData::Positional(mut param) => {
                                 line_idx +=
-                                    take_comment_lines(&lines, line_idx + 1, &mut param.describe);
+                                    take_comment_lines(&lines, line_idx + 1, param.describe_mut());
                                 EventData::Positional(param)
                             }
                             v => v,
@@ -239,11 +240,14 @@ fn parse_with_long_option_param(input: &str) -> nom::IResult<&str, FlagOptionPar
                 parse_param_assign,
                 parse_param_modifer,
             )),
+            parse_zero_or_one_bind_env,
             parse_zero_or_many_value_notations,
             parse_tail,
         )),
-        |((short, long_prefix), arg, value_names, describe)| {
-            FlagOptionParam::new(arg, describe, false, short, long_prefix, &value_names)
+        |((short, long_prefix), mut arg, env, value_names, describe)| {
+            arg.env = env;
+            arg.describe = describe.to_string();
+            FlagOptionParam::new(arg, false, short, long_prefix, &value_names)
         },
     )(input)
 }
@@ -264,11 +268,14 @@ fn parse_no_long_option_param(input: &str) -> nom::IResult<&str, FlagOptionParam
                     parse_param_modifer,
                 )),
             ),
+            parse_zero_or_one_bind_env,
             parse_zero_or_many_value_notations,
             parse_tail,
         )),
-        |(long_prefix, arg, value_names, describe)| {
-            FlagOptionParam::new(arg, describe, false, None, long_prefix, &value_names)
+        |(long_prefix, mut arg, env, value_names, describe)| {
+            arg.env = env;
+            arg.describe = describe.to_string();
+            FlagOptionParam::new(arg, false, None, long_prefix, &value_names)
         },
     )(input)
 }
@@ -287,7 +294,10 @@ fn parse_env_param(input: &str) -> nom::IResult<&str, EnvParam> {
             )),
             parse_tail,
         ),
-        |(arg, describe)| EnvParam::new(arg, describe),
+        |(mut arg, describe)| {
+            arg.describe = describe.to_string();
+            EnvParam::new(arg)
+        },
     )(input)
 }
 
@@ -303,10 +313,15 @@ fn parse_positional_param(input: &str) -> nom::IResult<&str, PositionalParam> {
                 parse_param_assign,
                 parse_param_modifer,
             )),
+            parse_zero_or_one_bind_env,
             parse_zero_or_one_value_notation,
             parse_tail,
         )),
-        |(arg, value_name, describe)| PositionalParam::new(arg, describe, value_name),
+        |(mut arg, env, value_name, describe)| {
+            arg.env = env;
+            arg.describe = describe.to_string();
+            PositionalParam::new(arg, value_name)
+        },
     )(input)
 }
 
@@ -318,9 +333,16 @@ fn parse_flag_param(input: &str) -> nom::IResult<&str, FlagOptionParam> {
 // Parse `@flag`
 fn parse_with_long_flag_param(input: &str) -> nom::IResult<&str, FlagOptionParam> {
     map(
-        tuple((parse_with_long_head, parse_with_long_flag_name, parse_tail)),
-        |((short, long_prefix), arg, describe)| {
-            FlagOptionParam::new(arg, describe, true, short, long_prefix, &[])
+        tuple((
+            parse_with_long_head,
+            parse_with_long_flag_name,
+            parse_zero_or_one_bind_env,
+            parse_tail,
+        )),
+        |((short, long_prefix), mut arg, env, describe)| {
+            arg.env = env;
+            arg.describe = describe.to_string();
+            FlagOptionParam::new(arg, true, short, long_prefix, &[])
         },
     )(input)
 }
@@ -331,10 +353,13 @@ fn parse_no_long_flag_param(input: &str) -> nom::IResult<&str, FlagOptionParam>
         tuple((
             preceded(space0, alt((tag("-"), tag("+")))),
             parse_no_long_flag_name,
+            parse_zero_or_one_bind_env,
             parse_tail,
         )),
-        |(long_prefix, arg, describe)| {
-            FlagOptionParam::new(arg, describe, true, None, long_prefix, &[])
+        |(long_prefix, mut arg, env, describe)| {
+            arg.env = env;
+            arg.describe = describe.to_string();
+            FlagOptionParam::new(arg, true, None, long_prefix, &[])
         },
     )(input)
 }
@@ -395,14 +420,7 @@ fn parse_param_modifer(input: &str) -> nom::IResult<&str, ParamData> {
             |(mut arg, multi_char)| {
                 match multi_char {
                     Some(c) => arg.modifer = Modifier::DelimiterOptional(c),
-                    None => {
-                        if let Some(name) = arg.name.strip_suffix('-') {
-                            arg.name = name.to_string();
-                            arg.modifer = Modifier::MultiPrefixed
-                        } else {
-                            arg.modifer = Modifier::MultipleOptional
-                        }
-                    }
+                    None => arg.modifer = Modifier::MultipleOptional,
                 };
                 arg
             },
@@ -418,13 +436,7 @@ fn parse_param_modifer(input: &str) -> nom::IResult<&str, ParamData> {
                 arg
             },
         ),
-        map(parse_param_name, |mut arg| {
-            if let Some(name) = arg.name.strip_suffix('-') {
-                arg.name = name.to_string();
-                arg.modifer = Modifier::Prefixed;
-            }
-            arg
-        }),
+        parse_param_name,
     ))(input)
 }
 
@@ -518,6 +530,10 @@ fn parse_value_notation(input: &str) -> nom::IResult<&str, &str> {
     )(input)
 }
 
+fn parse_bind_env_name(input: &str) -> nom::IResult<&str, &str> {
+    take_while1(is_env_name_char)(input)
+}
+
 // Parse `a|b|c`
 fn parse_choices(input: &str) -> nom::IResult<&str, Vec<&str>> {
     map(separated_list1(char('|'), parse_choice_value), |choices| {
@@ -661,6 +677,23 @@ fn parse_symbol_data(input: &str) -> nom::IResult<&str, (char, &str, Option<&str
     )(input)
 }
 
+fn parse_zero_or_one_bind_env(input: &str) -> nom::IResult<&str, Option<Option<String>>> {
+    opt(parse_bind_env)(input)
+}
+
+fn parse_bind_env(input: &str) -> nom::IResult<&str, Option<String>> {
+    map(
+        preceded(tag(" $"), alt((tag("$"), parse_bind_env_name))),
+        |v| {
+            if v == "$" {
+                None
+            } else {
+                Some(v.to_string())
+            }
+        },
+    )(input)
+}
+
 fn notation_text(input: &str, balances: usize) -> nom::IResult<&str, usize> {
     let (i1, c1) = anychar(input)?;
     match c1 {
@@ -696,8 +729,20 @@ fn create_err(input: &str, kind: ErrorKind) -> nom::Err<nom::error::Error<&str>>
     nom::Err::Error(nom::error::Error::new(input, kind))
 }
 
+pub(crate) fn is_choice_value_terminate(c: char) -> bool {
+    c == '|' || c == ']'
+}
+
+pub(crate) fn is_default_value_terminate(c: char) -> bool {
+    c.is_whitespace()
+}
+
 fn is_name_char(c: char) -> bool {
-    c.is_ascii_alphanumeric() || matches!(c, '_' | '-' | '.' | ':' | '@')
+    c.is_ascii_alphanumeric() || c == '_' || is_special_var_char(c)
+}
+
+fn is_env_name_char(c: char) -> bool {
+    c.is_ascii_uppercase() || c == '_'
 }
 
 fn is_short_char(c: char) -> bool {
@@ -859,6 +904,10 @@ mod tests {
         assert_parse_option_arg!("--foo!");
         assert_parse_option_arg!("--foo-*");
         assert_parse_option_arg!("--foo-");
+        assert_parse_option_arg!("--foo--");
+        assert_parse_option_arg!("--foo:*");
+        assert_parse_option_arg!("--foo:");
+        assert_parse_option_arg!("--foo::");
         assert_parse_option_arg!("--foo=a");
         assert_parse_option_arg!("--foo=`_foo`");
         assert_parse_option_arg!("--foo[a|b]");
@@ -884,17 +933,28 @@ mod tests {
         assert_parse_option_arg!("--foo <>");
         assert_parse_option_arg!("--foo <abc def>");
         assert_parse_option_arg!("--foo <<abc def>>");
+        assert_parse_option_arg!("--foo $$");
+        assert_parse_option_arg!("--foo $FOO");
+        assert_parse_option_arg!("--foo $FOO <FOO>");
     }
 
     #[test]
     fn test_parse_with_long_option_arg_single_dash() {
         assert_parse_option_arg!("-f -foo=a <FOO> A foo option");
+        assert_parse_option_arg!("-foo:bar");
+        assert_parse_option_arg!("-foo.bar");
         assert_parse_option_arg!("-foo!");
         assert_parse_option_arg!("-foo+");
         assert_parse_option_arg!("-foo*");
+        assert_parse_option_arg!("-foo+,");
+        assert_parse_option_arg!("-foo*,");
         assert_parse_option_arg!("-foo!");
         assert_parse_option_arg!("-foo-*");
         assert_parse_option_arg!("-foo-");
+        assert_parse_option_arg!("-foo--");
+        assert_parse_option_arg!("-foo:*");
+        assert_parse_option_arg!("-foo:");
+        assert_parse_option_arg!("-foo::");
         assert_parse_option_arg!("-foo=a");
         assert_parse_option_arg!("-foo=`_foo`");
         assert_parse_option_arg!("-foo[a|b]");
@@ -918,6 +978,9 @@ mod tests {
         assert_parse_option_arg!("-foo <>");
         assert_parse_option_arg!("-foo <abc def>");
         assert_parse_option_arg!("-foo <<abc def>>");
+        assert_parse_option_arg!("-foo $$");
+        assert_parse_option_arg!("-foo $FOO");
+        assert_parse_option_arg!("-foo $FOO <FOO>");
     }
 
     #[test]
@@ -1009,6 +1072,8 @@ mod tests {
         assert_parse_positional_arg!("foo*[a|b]");
         assert_parse_positional_arg!("foo*[`_foo`]");
         assert_parse_positional_arg!("foo*[=a|b]");
+        assert_parse_positional_arg!("foo $$");
+        assert_parse_positional_arg!("foo $FOO");
     }
 
     #[test]
