@@ -1,7 +1,7 @@
 use convert_case::{Boundary, Converter, Pattern};
 use std::{
     collections::HashMap,
-    env,
+    env, fs,
     path::{Path, PathBuf},
     process, thread,
 };
@@ -14,15 +14,30 @@ pub const AFTER_HOOK: &str = "_argc_after";
 pub const ROOT_NAME: &str = "prog";
 pub const MAIN_NAME: &str = "main";
 
+pub(crate) const META_VERSION: &str = "version";
+pub(crate) const META_AUTHOR: &str = "author";
 pub(crate) const META_DOTENV: &str = "dotenv";
 pub(crate) const META_DEFAULT_SUBCOMMAND: &str = "default-subcommand";
 pub(crate) const META_INHERIT_FLAG_OPTIONS: &str = "inherit-flag-options";
-pub(crate) const META_NO_INHERIT_ENV: &str = "no-inherit-env";
 pub(crate) const META_SYMBOL: &str = "symbol";
 pub(crate) const META_COMBINE_SHORTS: &str = "combine-shorts";
 pub(crate) const META_MAN_SECTION: &str = "man-section";
+pub(crate) const META_REQUIRE_TOOLS: &str = "require-tools";
 
-pub(crate) const MAX_ARGS: usize = 999999;
+pub(crate) const MAX_ARGS: usize = 32767;
+
+pub(crate) const ARGC_REQUIRE_TOOLS: &str = r#"_argc_require_tools() {
+    local tool missing_tools=()
+    for tool in "$@"; do
+        if ! command -v "$tool" >/dev/null 2>&1; then
+            missing_tools+=("$tool")
+        fi
+    done
+    if [[ "${#missing_tools[@]}" -gt 0 ]]; then
+        echo "error: missing tools: ${missing_tools[*]}" >&2
+        exit 1
+    fi
+}"#;
 
 pub fn to_cobol_case(value: &str) -> String {
     Converter::new()
@@ -36,18 +51,10 @@ pub fn escape_shell_words(value: &str) -> String {
     shell_words::quote(value).to_string()
 }
 
-pub fn is_choice_value_terminate(c: char) -> bool {
-    c == '|' || c == ']'
-}
-
 pub fn is_quote_char(c: char) -> bool {
     c == '\'' || c == '"'
 }
 
-pub fn is_default_value_terminate(c: char) -> bool {
-    c.is_whitespace()
-}
-
 pub fn get_shell_path() -> anyhow::Result<PathBuf> {
     match env::var("ARGC_SHELL_PATH") {
         Ok(v) => {
@@ -78,6 +85,10 @@ pub fn get_shell_args(shell_path: &Path) -> Vec<String> {
     }
 }
 
+pub fn get_os() -> String {
+    env::consts::OS.to_string()
+}
+
 #[cfg(windows)]
 pub fn get_bash_path() -> Option<PathBuf> {
     let bash_path = PathBuf::from("C:\\Program Files\\Git\\bin\\bash.exe");
@@ -186,18 +197,46 @@ pub fn path_env_with_exe() -> String {
 }
 
 pub fn expand_dotenv(value: &str) -> String {
-    let value = if value.is_empty() { ".env" } else { value };
+    let value = escape_shell_words(value);
     format!("[ -f {value} ] && set -o allexport && . {value} && set +o allexport")
 }
 
+pub fn is_special_var_char(c: char) -> bool {
+    matches!(c, '-' | '.' | ':' | '@')
+}
+
 pub fn sanitize_var_name(id: &str) -> String {
-    id.replace(['-', '.', ':'], "_")
+    id.replace(is_special_var_char, "_")
 }
 
 pub fn argc_var_name(id: &str) -> String {
     format!("{VARIABLE_PREFIX}{}", sanitize_var_name(id))
 }
 
+pub fn all_envs() -> HashMap<String, String> {
+    env::vars().collect()
+}
+
+pub fn load_dotenv(path: &str) -> Option<HashMap<String, String>> {
+    let contents = fs::read_to_string(path).ok()?;
+    let mut output = HashMap::new();
+    for line in contents.lines() {
+        if line.starts_with('#') || line.trim().is_empty() {
+            continue;
+        }
+        if let Some((key, value)) = line.split_once('=') {
+            let key = key.trim().to_string();
+            let value = value.trim().to_string();
+            output.insert(key, value);
+        }
+    }
+    Some(output)
+}
+
+pub fn is_true_value(value: &str) -> bool {
+    matches!(value, "true" | "1")
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
