@@ -224,8 +224,34 @@ fn option_prefixed() {
     let script = r###"
 # @option -o-
 # @option -D-*
+# @flag -v--
 "###;
-    snapshot!(script, &["prog", "-o1", "-Dv1=foo", "-Dv2", "bar"]);
+    snapshot_multi!(
+        script,
+        [
+            vec!["prog", "-o1", "-Dv1=foo", "-Dv2", "bar"],
+            vec!["prog", "-v-"]
+        ]
+    );
+}
+
+#[test]
+fn option_assigned() {
+    let script = r###"
+# @option --oa:
+# @option --ob:*
+# @option --oc: <VALUE?>
+# @option --o::
+"###;
+    snapshot_multi!(
+        script,
+        [
+            vec!["prog", "--oa=v1", "--ob=1", "--ob=2"],
+            vec!["prog", "--oa", "v1"],
+            vec!["prog", "--oc", "v1"],
+            vec!["prog", "--o:", "v1"],
+        ]
+    );
 }
 
 #[test]
@@ -345,6 +371,8 @@ fn default_subcommand() {
     let script = r###"
 # @cmd
 # @meta default-subcommand
+# @flag --fa
+# @arg val
 cmda() { :; }
 
 # @cmd
@@ -354,6 +382,8 @@ cmdb() { :; }
         script,
         [
             vec!["prog", "-h"],
+            vec!["prog", "--fa"],
+            vec!["prog", "--fa", "-h"],
             vec!["prog", "v1"],
             vec!["prog", "cmda", "v1"],
             vec!["prog", "cmdb", "v1"],
